#! python
# Python Serial Port Extension for Win32, Linux, BSD, Jython
# see __init__.py
#
# (C) 2001-2008 Chris Liechti <cliechti@gmx.net>
# this is distributed under a free software license, see license.txt

PARITY_NONE, PARITY_EVEN, PARITY_ODD, PARITY_MARK, PARITY_SPACE = 'N', 'E', 'O', 'M', 'S'
STOPBITS_ONE, STOPBITS_TWO = (1, 2)
FIVEBITS, SIXBITS, SEVENBITS, EIGHTBITS = (5, 6, 7, 8)

PARITY_NAMES = {
    PARITY_NONE: 'None',
    PARITY_EVEN: 'Even',
    PARITY_ODD: 'Odd',
    PARITY_MARK: 'Mark',
    PARITY_SPACE: 'Space',
}

XON = chr(17)
XOFF = chr(19)

# Python < 2.2.3 compatibility
try:
    True
except BaseException:
    True = 1
    False = not True


class SerialException(Exception):

    """Base class for serial port related exceptions."""


portNotOpenError = SerialException('Port not open')


class SerialTimeoutException(SerialException):

    """Write timeouts give an exception"""


writeTimeoutError = SerialTimeoutException("Write timeout")


class FileLike(object):

    """An abstract file like class.

    This class implements readline and readlines based on read and
    writelines based on write.
    This class is used to provide the above functions for to Serial
    port objects.

    Note that when the serial port was opened with _NO_ timeout that
    readline blocks until it sees a newline (or the specified size is
    reached) and that readlines would never return and therefore
    refuses to work (it raises an exception in this case)!
    """

    def read(self, size):
        raise NotImplementedError

    def write(self, s):
        raise NotImplementedError

    def readline(self, size=None, eol='\n'):
        """read a line which is terminated with end-of-line (eol) character
        ('\n' by default) or until timeout"""
        line = ''
        while True:
            c = self.read(1)
            if c:
                # not very efficient but lines are usually not that long
                line += c
                if c == eol:
                    break
                if size is not None and len(line) >= size:
                    break
            else:
                break
        return line

    def readlines(self, sizehint=None, eol='\n'):
        """read a list of lines, until timeout
        sizehint is ignored"""
        if self.timeout is None:
            raise ValueError(
                "Serial port MUST have enabled timeout for this function!")
        lines = []
        while True:
            line = self.readline(eol=eol)
            if line:
                lines.append(line)
                if line[-1] != eol:  # was the line received with a timeout?
                    break
            else:
                break
        return lines

    def xreadlines(self, sizehint=None):
        """just call readlines - here for compatibility"""
        return self.readlines()

    def writelines(self, sequence):
        for line in sequence:
            self.write(line)

    def flush(self):
        """flush of file like objects"""
        pass

    # iterator for e.g. "for line in Serial(0): ..." usage
    def next(self):
        line = self.readline()
        if not line:
            raise StopIteration
        return line

    def __iter__(self):
        return self


class SerialBase(FileLike):

    """Serial port base class. Provides __init__ function and properties to
       get/set port settings."""

    # default values, may be overriden in subclasses that do not support all
    # values
    BAUDRATES = (
        50,
        75,
        110,
        134,
        150,
        200,
        300,
        600,
        1200,
        1800,
        2400,
        4800,
        9600,
        19200,
        38400,
        57600,
        115200,
        230400,
        460800,
        500000,
        576000,
        921600,
        1000000,
        1152000,
        1500000,
        2000000,
        2500000,
        3000000,
        3500000,
        4000000)
    BYTESIZES = (FIVEBITS, SIXBITS, SEVENBITS, EIGHTBITS)
    PARITIES = (PARITY_NONE, PARITY_EVEN, PARITY_ODD)
    STOPBITS = (STOPBITS_ONE, STOPBITS_TWO)

    def __init__(self,
                 port=None,  # number of device, numbering starts at
                 # zero. if everything fails, the user
                 # can specify a device string, note
                 # that this isn't portable anymore
                 # port will be opened if one is specified
                 baudrate=9600,  # baudrate
                 bytesize=EIGHTBITS,  # number of databits
                 parity=PARITY_NONE,  # enable parity checking
                 stopbits=STOPBITS_ONE,  # number of stopbits
                 timeout=None,  # set a timeout value, None to wait forever
                 xonxoff=0,  # enable software flow control
                 rtscts=0,  # enable RTS/CTS flow control
                 writeTimeout=None,  # set a timeout for writes
                 dsrdtr=None,
                 # None: use rtscts setting, dsrdtr override if true or false
                 # Inter-character timeout, None to disable
                 interCharTimeout=None
                 ):
        """Initialize comm port object. If a port is given, then the port will be
           opened immediately. Otherwise a Serial port object in closed state
           is returned."""

        self._isOpen = False
        self._port = None  # correct value is assigned below trough properties
        # correct value is assigned below trough properties
        self._baudrate = None
        # correct value is assigned below trough properties
        self._bytesize = None
        # correct value is assigned below trough properties
        self._parity = None
        # correct value is assigned below trough properties
        self._stopbits = None
        # correct value is assigned below trough properties
        self._timeout = None
        # correct value is assigned below trough properties
        self._writeTimeout = None
        # correct value is assigned below trough properties
        self._xonxoff = None
        # correct value is assigned below trough properties
        self._rtscts = None
        # correct value is assigned below trough properties
        self._dsrdtr = None
        # correct value is assigned below trough properties
        self._interCharTimeout = None

        # assign values using get/set methods using the properties feature
        self.port = port
        self.baudrate = baudrate
        self.bytesize = bytesize
        self.parity = parity
        self.stopbits = stopbits
        self.timeout = timeout
        self.writeTimeout = writeTimeout
        self.xonxoff = xonxoff
        self.rtscts = rtscts
        self.dsrdtr = dsrdtr
        self.interCharTimeout = interCharTimeout

        if port is not None:
            self.open()

    def isOpen(self):
        """Check if the port is opened."""
        return self._isOpen

    #  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -

    # TODO: these are not realy needed as the is the BAUDRATES etc attribute...
    # maybe i remove them before the final release...

    def getSupportedBaudrates(self):
        return [(str(b), b) for b in self.BAUDRATES]

    def getSupportedByteSizes(self):
        return [(str(b), b) for b in self.BYTESIZES]

    def getSupportedStopbits(self):
        return [(str(b), b) for b in self.STOPBITS]

    def getSupportedParities(self):
        return [(PARITY_NAMES[b], b) for b in self.PARITIES]

    #  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -

    def setPort(self, port):
        """Change the port. The attribute portstr is set to a string that
           contains the name of the port."""

        was_open = self._isOpen
        if was_open:
            self.close()
        if port is not None:
            if type(port) in [
                    type(''),
                    type(u'')]:  # strings are taken directly
                self.portstr = port
            else:
                self.portstr = self.makeDeviceName(port)
        else:
            self.portstr = None
        self._port = port
        if was_open:
            self.open()

    def getPort(self):
        """Get the current port setting. The value that was passed on init or using
           setPort() is passed back. See also the attribute portstr which contains
           the name of the port as a string."""
        return self._port

    port = property(getPort, setPort, doc="Port setting")

    def setBaudrate(self, baudrate):
        """Change baudrate. It raises a ValueError if the port is open and the
        baudrate is not possible. If the port is closed, then tha value is
        accepted and the exception is raised when the port is opened."""
        #~ if baudrate not in self.BAUDRATES: raise ValueError("Not a valid baudrate: %r" % baudrate)
        try:
            self._baudrate = int(baudrate)
        except TypeError:
            raise ValueError("Not a valid baudrate: %r" % (baudrate,))
        else:
            if self._isOpen:
                self._reconfigurePort()

    def getBaudrate(self):
        """Get the current baudrate setting."""
        return self._baudrate

    baudrate = property(getBaudrate, setBaudrate, doc="Baudrate setting")

    def setByteSize(self, bytesize):
        """Change byte size."""
        if bytesize not in self.BYTESIZES:
            raise ValueError("Not a valid byte size: %r" % (bytesize,))
        self._bytesize = bytesize
        if self._isOpen:
            self._reconfigurePort()

    def getByteSize(self):
        """Get the current byte size setting."""
        return self._bytesize

    bytesize = property(getByteSize, setByteSize, doc="Byte size setting")

    def setParity(self, parity):
        """Change parity setting."""
        if parity not in self.PARITIES:
            raise ValueError("Not a valid parity: %r" % (parity,))
        self._parity = parity
        if self._isOpen:
            self._reconfigurePort()

    def getParity(self):
        """Get the current parity setting."""
        return self._parity

    parity = property(getParity, setParity, doc="Parity setting")

    def setStopbits(self, stopbits):
        """Change stopbits size."""
        if stopbits not in self.STOPBITS:
            raise ValueError("Not a valid stopbit size: %r" % (stopbits,))
        self._stopbits = stopbits
        if self._isOpen:
            self._reconfigurePort()

    def getStopbits(self):
        """Get the current stopbits setting."""
        return self._stopbits

    stopbits = property(getStopbits, setStopbits, doc="Stopbits setting")

    def setTimeout(self, timeout):
        """Change timeout setting."""
        if timeout is not None:
            if timeout < 0:
                raise ValueError("Not a valid timeout: %r" % (timeout,))
            try:
                # test if it's a number, will throw a TypeError if not...
                timeout + 1
            except TypeError:
                raise ValueError("Not a valid timeout: %r" % (timeout,))

        self._timeout = timeout
        if self._isOpen:
            self._reconfigurePort()

    def getTimeout(self):
        """Get the current timeout setting."""
        return self._timeout

    timeout = property(
        getTimeout,
        setTimeout,
        doc="Timeout setting for read()")

    def setWriteTimeout(self, timeout):
        """Change timeout setting."""
        if timeout is not None:
            if timeout < 0:
                raise ValueError("Not a valid timeout: %r" % (timeout,))
            try:
                # test if it's a number, will throw a TypeError if not...
                timeout + 1
            except TypeError:
                raise ValueError("Not a valid timeout: %r" % timeout)

        self._writeTimeout = timeout
        if self._isOpen:
            self._reconfigurePort()

    def getWriteTimeout(self):
        """Get the current timeout setting."""
        return self._writeTimeout

    writeTimeout = property(
        getWriteTimeout,
        setWriteTimeout,
        doc="Timeout setting for write()")

    def setXonXoff(self, xonxoff):
        """Change XonXoff setting."""
        self._xonxoff = xonxoff
        if self._isOpen:
            self._reconfigurePort()

    def getXonXoff(self):
        """Get the current XonXoff setting."""
        return self._xonxoff

    xonxoff = property(getXonXoff, setXonXoff, doc="Xon/Xoff setting")

    def setRtsCts(self, rtscts):
        """Change RtsCts flow control setting."""
        self._rtscts = rtscts
        if self._isOpen:
            self._reconfigurePort()

    def getRtsCts(self):
        """Get the current RtsCts flow control setting."""
        return self._rtscts

    rtscts = property(getRtsCts, setRtsCts, doc="RTS/CTS flow control setting")

    def setDsrDtr(self, dsrdtr=None):
        """Change DsrDtr flow control setting."""
        if dsrdtr is None:
            # if not set, keep backwards compatibility and follow rtscts
            # setting
            self._dsrdtr = self._rtscts
        else:
            # if defined independently, follow its value
            self._dsrdtr = dsrdtr
        if self._isOpen:
            self._reconfigurePort()

    def getDsrDtr(self):
        """Get the current DsrDtr flow control setting."""
        return self._dsrdtr

    dsrdtr = property(getDsrDtr, setDsrDtr, "DSR/DTR flow control setting")

    def setInterCharTimeout(self, interCharTimeout):
        """Change inter-character timeout setting."""
        if interCharTimeout is not None:
            if interCharTimeout < 0:
                raise ValueError("Not a valid timeout: %r" % interCharTimeout)
            try:
                # test if it's a number, will throw a TypeError if not...
                interCharTimeout + 1
            except TypeError:
                raise ValueError("Not a valid timeout: %r" % interCharTimeout)

        self._interCharTimeout = interCharTimeout
        if self._isOpen:
            self._reconfigurePort()

    def getInterCharTimeout(self):
        """Get the current inter-character timeout setting."""
        return self._interCharTimeout

    interCharTimeout = property(
        getInterCharTimeout,
        setInterCharTimeout,
        doc="Inter-character timeout setting for read()")

    #  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -

    def __repr__(self):
        """String representation of the current port settings and its state."""
        return "%s<id=0x%x, open=%s>(port=%r, baudrate=%r, bytesize=%r, parity=%r, stopbits=%r, timeout=%r, xonxoff=%r, rtscts=%r, dsrdtr=%r)" % (
            self.__class__.__name__,
            id(self),
            self._isOpen,
            self.portstr,
            self.baudrate,
            self.bytesize,
            self.parity,
            self.stopbits,
            self.timeout,
            self.xonxoff,
            self.rtscts,
            self.dsrdtr,
        )


if __name__ == '__main__':
    s = SerialBase()
    print s.portstr
    print s.getSupportedBaudrates()
    print s.getSupportedByteSizes()
    print s.getSupportedParities()
    print s.getSupportedStopbits()
    print s



# Copyright (c) 2013 Walter Bender
# Copyright (c) 2013 Daniel Francis

# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:

# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.

# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.

import configparser
from gettext import gettext as _
import os
import shutil
import subprocess
from TurtleArt.tapalette import (palette_names, help_strings)


def cancel_plugin_install(tmp_dir):
    ''' If we cancel, just cleanup '''
    shutil.rmtree(tmp_dir)


def complete_plugin_install(cls, tmp_dir, tmp_path, plugin_path,
                            plugin_name, file_info):
    ''' We complete the installation directly or from ConfirmationAlert '''
    status = subprocess.call(['cp', '-r', tmp_path, plugin_path + '/'])
    if status == 0:
        # Save the plugin.info file in the plugin directory
        subprocess.call(['cp', os.path.join(tmp_dir, 'plugin.info'),
                         os.path.join(plugin_path, plugin_name) + '/'])

        palette_name_list = []
        if file_info.has_option('Plugin', 'palette'):
            palette_name_list = file_info.get(
                'Plugin', 'palette').split(',')
            create_palette = []
            for palette_name in palette_name_list:
                if not palette_name.strip() in palette_names:
                    create_palette.append(True)
                else:
                    create_palette.append(False)
        cls.tw.init_plugin(plugin_name)
        cls.tw.turtleart_plugins[-1].setup()
        cls.tw.load_media_shapes()
        for i, palette_name in enumerate(palette_name_list):
            if create_palette[i]:
                j = len(cls.palette_buttons)
                cls.palette_buttons.append(
                    cls._radio_button_factory(
                        palette_name.strip() + 'off',
                        cls._palette_toolbar,
                        cls.do_palette_buttons_cb,
                        j - 1,
                        help_strings[palette_name.strip()],
                        cls.palette_buttons[0]))
                cls._overflow_buttons.append(
                    cls._add_button(
                        palette_name.strip() + 'off',
                        None,
                        cls.do_palette_buttons_cb,
                        None,
                        arg=j - 1))
                cls._overflow_box.pack_start(
                    cls._overflow_buttons[j - 1])
                cls.tw.palettes.insert(j - 1, [])
                cls.tw.palette_sprs.insert(j - 1, [None, None])
            else:
                # We need to change the index associated with the
                # Trash Palette Button.
                j = len(palette_names)
                pidx = palette_names.index(palette_name.strip())
                cls.palette_buttons[pidx].connect(
                    'clicked', cls.do_palette_buttons_cb, j - 1)
                cls._overflow_buttons[pidx].connect(
                    'clicked', cls.do_palette_buttons_cb, j - 1)
                cls._setup_palette_toolbar()
        cls.tw.showlabel(
            'status',
            _('Please restart %s in order to use the plugin.') % cls.name)

    else:
        cls.tw.showlabel('status', _('Plugin could not be installed.'))
    status = subprocess.call(['rm', '-r', tmp_path])
    shutil.rmtree(tmp_dir)


def load_a_plugin(cls, tmp_dir):
    ''' Load a plugin from the Journal and initialize it '''
    plugin_path = os.path.join(tmp_dir, 'plugin.info')
    file_info = configparser.ConfigParser()
    if len(file_info.read(plugin_path)) == 0:
        cls.tw.showlabel('status', _('Plugin could not be installed.'))
    elif not file_info.has_option('Plugin', 'name'):
        cls.tw.showlabel('status', _('Plugin could not be installed.'))
    else:
        plugin_name = file_info.get('Plugin', 'name')
        tmp_path = os.path.join(tmp_dir, plugin_name)
        plugin_path = os.path.join(cls.bundle_path, 'plugins')
        if os.path.exists(os.path.join(plugin_path, plugin_name)):
            cls._reload_plugin_alert(tmp_dir, tmp_path, plugin_path,
                                     plugin_name, file_info)
        else:
            complete_plugin_install(cls, tmp_dir, tmp_path, plugin_path,
                                    plugin_name, file_info)



# Copyright (c) 2009-11, Walter Bender

# This procedure is invoked when the user-definable block on the
# "extras" palette is selected and expanded to 3 arguments.

# Usage: Import this code into a Python (user-definable) block.
# First, expand the Python block to reveal three numerics arguments.
# Set these values to the desired red, green, and blue. When the code
# is run, the red, green, and blue values are used to set the pen
# color.


def myblock(tw, rgb_array):
    ''' Set rgb color from values '''

    tw.canvas._fgrgb = rgb_array[:]



# -*- coding: utf-8 -*-

# Copyright (c) 2007-8, Playful Invention Company.
# Copyright (c) 2008-14 Walter Bender

# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:

# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.

# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.

'''

sprites.py is a simple sprites library for managing graphics objects,
'sprites', on a gtk.DrawingArea. It manages multiple sprites with
methods such as move, hide, set_layer, etc.

There are two classes:

class Sprites maintains a collection of sprites
class Sprite manages individual sprites within the collection.

Example usage:
        # Import the classes into your program.
        from sprites import Sprites Sprite

        # Create a new sprite collection associated with your widget
        self.sprite_list = Sprites(widget)

        # Create a "pixbuf" (in this example, from SVG).
        my_pixbuf = svg_str_to_pixbuf("<svg>...some svg code...</svg>")

        # Create a sprite at position x1, y1.
        my_sprite = sprites.Sprite(self.sprite_list, x1, y1, my_pixbuf)

        # Move the sprite to a new position.
        my_sprite.move((x1+dx, y1+dy))

        # Create another "pixbuf".
        your_pixbuf = svg_str_to_pixbuf("<svg>...some svg code...</svg>")

        # Create a sprite at position x2, y2.
        your_sprite = sprites.Sprite(self.sprite_list, x2, y2, my_pixbuf)

        # Assign the sprites to layers.
        # In this example, your_sprite will be on top of my_sprite.
        my_sprite.set_layer(100)
        your_sprite.set_layer(200)

        # Now put my_sprite on top of your_sprite.
        my_sprite.set_layer(300)

        cr = self.window.cairo_create()
        # In your activity's do_expose_event, put in a call to redraw_sprites
        self.sprites.redraw_sprites(event.area, cairo_context)

# method for converting SVG to a gtk pixbuf
def svg_str_to_pixbuf(svg_string):
    pl = gtk.gdk.PixbufLoader('svg')
    pl.write(svg_string)
    pl.close()
    pixbuf = pl.get_pixbuf()
    return pixbuf

'''
import cairo

from gi.repository import Gdk
from gi.repository import GdkPixbuf
from gi.repository import Pango
from gi.repository import PangoCairo


class Sprites:

    ''' A class for the list of sprites and everything they share in common '''

    def __init__(self, widget):
        ''' Initialize an empty array of sprites '''
        self.widget = widget
        self.list = []
        self.cr = None
        self.defer_draw = False

    def set_defer_draw(self, state):
        self.defer_draw = state

    def set_cairo_context(self, cr):
        ''' Cairo context may be set or reset after __init__ '''
        self.cr = cr

    def get_sprite(self, i):
        ''' Return a sprint from the array '''
        if i < 0 or i > len(self.list) - 1:
            return(None)
        else:
            return(self.list[i])

    def length_of_list(self):
        ''' How many sprites are there? '''
        return len(self.list)

    def append_to_list(self, spr):
        ''' Append a new sprite to the end of the list. '''
        self.list.append(spr)

    def insert_in_list(self, spr, i):
        ''' Insert a sprite at position i. '''
        if i < 0:
            self.list.insert(0, spr)
        elif i > len(self.list) - 1:
            self.list.append(spr)
        else:
            self.list.insert(i, spr)

    def find_in_list(self, spr):
        return (spr in self.list)

    def remove_from_list(self, spr):
        ''' Remove a sprite from the list. '''
        if spr in self.list:
            self.list.remove(spr)

    def find_sprite(self, pos, region=False):
        ''' Search based on (x, y) position. Return the 'top/first' one. '''
        list = self.list[:]
        list.reverse()
        for spr in list:
            if spr.hit(pos, readpixel=not region):
                return spr
        return None

    def redraw_sprites(self, area=None, cr=None):
        ''' Redraw the sprites that intersect area. '''
        # I think I need to do this to save Cairo some work
        self.defer_draw = False
        if cr is None:
            cr = self.cr
        else:
            self.cr = cr
        if cr is None:
            print('sprites.redraw_sprites: no Cairo context')
            return
        for spr in self.list:
            if area is None:
                spr.draw(cr=cr)
            else:
                intersection = spr.rect.intersect(area)
                if intersection.width > 0 or intersection.height > 0:
                    spr.draw(cr=cr)


class Sprite:

    ''' A class for the individual sprites '''

    def __init__(self, sprites, x, y, image):
        ''' Initialize an individual sprite '''
        self._sprites = sprites
        self.save_xy = (x, y)  # remember initial (x, y) position
        self.rect = Gdk.Rectangle()
        self.rect.x = int(x)
        self.rect.y = int(y)
        self.rect.width = image.get_height()
        self.rect.height = image.get_width()
        self._scale = [12]
        self._rescale = [True]
        self._horiz_align = ['center']
        self._vert_align = ['middle']
        self._x_pos = [None]
        self._y_pos = [None]
        self._fd = None
        self._bold = False
        self._italic = False
        self._color = None
        self._margins = [0, 0, 0, 0]
        self.layer = 100
        self.labels = []
        self.cached_surfaces = []
        self._dx = []  # image offsets
        self._dy = []
        self.type = None
        self.set_image(image)
        self._sprites.append_to_list(self)

    def set_image(self, image, i=0, dx=0, dy=0):
        ''' Add an image to the sprite. '''
        while len(self.cached_surfaces) < i + 1:
            self.cached_surfaces.append(None)
            self._dx.append(0)
            self._dy.append(0)
        self._dx[i] = dx
        self._dy[i] = dy
        if isinstance(image, GdkPixbuf.Pixbuf) or \
           isinstance(image, cairo.ImageSurface):
            w = image.get_width()
            h = image.get_height()
        else:
            w, h = image.get_size()
        if i == 0:  # Always reset width and height when base image changes.
            self.rect.width = w + dx
            self.rect.height = h + dy
        else:
            if w + dx > self.rect.width:
                self.rect.width = w + dx
            if h + dy > self.rect.height:
                self.rect.height = h + dy
        if isinstance(image, cairo.ImageSurface):
            self.cached_surfaces[i] = image
        else:  # Convert to Cairo surface
            surface = cairo.ImageSurface(
                cairo.FORMAT_ARGB32, self.rect.width, self.rect.height)
            context = cairo.Context(surface)
            Gdk.cairo_set_source_pixbuf(context, image, 0, 0)
            context.rectangle(0, 0, self.rect.width, self.rect.height)
            context.fill()
            self.cached_surfaces[i] = surface

    def move(self, pos):
        ''' Move to new (x, y) position '''
        self.inval()
        self.rect.x, self.rect.y = int(pos[0]), int(pos[1])
        self.inval()

    def move_relative(self, pos):
        ''' Move to new (x+dx, y+dy) position '''
        self.inval()
        self.rect.x += int(pos[0])
        self.rect.y += int(pos[1])
        self.inval()

    def get_xy(self):
        ''' Return current (x, y) position '''
        return (self.rect.x, self.rect.y)

    def get_dimensions(self):
        ''' Return current size '''
        return (self.rect.width, self.rect.height)

    def get_layer(self):
        ''' Return current layer '''
        return self.layer

    def set_shape(self, image, i=0):
        ''' Set the current image associated with the sprite '''
        self.inval()
        self.set_image(image, i)
        self.inval()

    def set_layer(self, layer=None):
        ''' Set the layer for a sprite '''
        self._sprites.remove_from_list(self)
        if layer is not None:
            self.layer = layer
        for i in range(self._sprites.length_of_list()):
            spr = self._sprites.get_sprite(i)
            if spr is not None and self.layer < spr.layer:
                self._sprites.insert_in_list(self, i)
                self.inval()
                return
        self._sprites.append_to_list(self)
        self.inval()

    def set_label(self, new_label, i=0):
        ''' Set the label drawn on the sprite '''
        self._extend_labels_array(i)
        if isinstance(new_label, str):
            # pango doesn't like nulls
            self.labels[i] = new_label.replace('\0', ' ')
        else:
            self.labels[i] = str(new_label)
        self.inval()

    def set_margins(self, l=0, t=0, r=0, b=0):
        ''' Set the margins for drawing the label '''
        self._margins = [l, t, r, b]

    def _extend_labels_array(self, i):
        ''' Append to the labels attribute list '''
        if self._fd is None:
            self.set_font('Sans')
        if self._color is None:
            self._color = (0., 0., 0.)
        while len(self.labels) < i + 1:
            self.labels.append(' ')
            self._scale.append(self._scale[0])
            self._rescale.append(self._rescale[0])
            self._horiz_align.append(self._horiz_align[0])
            self._vert_align.append(self._vert_align[0])
            self._x_pos.append(self._x_pos[0])
            self._y_pos.append(self._y_pos[0])

    def set_font(self, font):
        ''' Set the font for a label '''
        self._fd = Pango.FontDescription(font)

    def set_label_color(self, rgb):
        ''' Set the font color for a label '''
        COLORTABLE = {'black': '#000000', 'white': '#FFFFFF',
                      'red': '#FF0000', 'yellow': '#FFFF00',
                      'green': '#00FF00', 'cyan': '#00FFFF',
                      'blue': '#0000FF', 'purple': '#FF00FF',
                      'gray': '#808080'}
        if rgb.lower() in COLORTABLE:
            rgb = COLORTABLE[rgb.lower()]
        # Convert from '#RRGGBB' to floats
        self._color = (int('0x' + rgb[1:3], 16) / 256.,
                       int('0x' + rgb[3:5], 16) / 256.,
                       int('0x' + rgb[5:7], 16) / 256.)
        return

    def set_label_attributes(self, scale, rescale=True, horiz_align='center',
                             vert_align='middle', x_pos=None, y_pos=None, i=0):
        ''' Set the various label attributes '''
        self._extend_labels_array(i)
        self._scale[i] = scale
        self._rescale[i] = rescale
        self._horiz_align[i] = horiz_align
        self._vert_align[i] = vert_align
        self._x_pos[i] = x_pos
        self._y_pos[i] = y_pos

    def hide(self):
        ''' Hide a sprite '''
        self.inval()
        self._sprites.remove_from_list(self)

    def restore(self):
        ''' Restore a hidden sprite '''
        self.set_layer()

    def inval(self):
        ''' Invalidate a region for gtk '''
        self._sprites.widget.queue_draw_area(self.rect.x,
                                             self.rect.y,
                                             self.rect.width,
                                             self.rect.height)

    def draw(self, cr=None):
        ''' Draw the sprite (and label) '''
        if self._sprites.defer_draw:
            return
        if cr is None:
            print('sprite.draw: no Cairo context.')
            return
        for i, surface in enumerate(self.cached_surfaces):
            cr.set_source_surface(surface,
                                  self.rect.x + self._dx[i],
                                  self.rect.y + self._dy[i])
            cr.rectangle(self.rect.x + self._dx[i],
                         self.rect.y + self._dy[i],
                         self.rect.width,
                         self.rect.height)
            cr.fill()

        if len(self.labels) > 0:
            self.draw_label(cr)

    def hit(self, pos, readpixel=False):
        ''' Is (x, y) on top of the sprite? '''
        x, y = pos
        if x < self.rect.x:
            return False
        if x > self.rect.x + self.rect.width:
            return False
        if y < self.rect.y:
            return False
        if y > self.rect.y + self.rect.height:
            return False
        if readpixel:
            r, g, b, a = self.get_pixel(pos)
            if r == g == b == a == 0:
                return False
            if a == -1:
                return False
        return self._sprites.find_in_list(self)

    def draw_label(self, cr):
        ''' Draw the label based on its attributes '''

        my_width = self.rect.width - self._margins[0] - self._margins[2]
        if my_width < 0:
            my_width = 0
        my_height = self.rect.height - self._margins[1] - self._margins[3]
        for i in range(len(self.labels)):
            pl = PangoCairo.create_layout(cr)
            pl.set_text(self.labels[i], -1)
            self._fd.set_size(int(self._scale[i] * Pango.SCALE))
            pl.set_font_description(self._fd)
            w = pl.get_size()[0] / Pango.SCALE
            if w > my_width:
                if self._rescale[i]:
                    self._fd.set_size(
                        int(self._scale[i] * Pango.SCALE * my_width / w))
                    pl.set_font_description(self._fd)
                    w = pl.get_size()[0] / Pango.SCALE
                else:
                    pl.set_ellipsize(Pango.EllipsizeMode.MIDDLE)
                    self._fd.set_size(int(self._scale[i] * Pango.SCALE))
                    pl.set_font_description(self._fd)
                    w = pl.get_size()[0] / Pango.SCALE
            if self._x_pos[i] is not None:
                x = int(self.rect.x + self._x_pos[i])
            elif self._horiz_align[i] == 'center':
                x = int(self.rect.x + self._margins[0] + (my_width - w) / 2)
            elif self._horiz_align[i] == 'left':
                x = int(self.rect.x + self._margins[0])
            else:  # right
                x = int(self.rect.x + self.rect.width - w - self._margins[2])
            h = pl.get_size()[1] / Pango.SCALE
            if self._y_pos[i] is not None:
                y = int(self.rect.y + self._y_pos[i])
            elif self._vert_align[i] == 'middle':
                y = int(self.rect.y + self._margins[1] + (my_height - h) / 2)
            elif self._vert_align[i] == 'top':
                y = int(self.rect.y + self._margins[1])
            else:  # bottom
                y = int(self.rect.y + self.rect.height - h - self._margins[3])

            cr.save()
            cr.translate(x, y)
            cr.set_source_rgb(self._color[0], self._color[1], self._color[2])
            PangoCairo.update_layout(cr, pl)
            PangoCairo.show_layout(cr, pl)

            cr.restore()

    def label_width(self):
        ''' Calculate the width of a label '''
        cr = self._sprites.cr
        if cr is not None:
            max = 0
            for i in range(len(self.labels)):
                pl = PangoCairo.create_layout(cr)
                pl.set_text(self.labels[i], -1)
                self._fd.set_size(int(self._scale[i] * Pango.SCALE))
                pl.set_font_description(self._fd)
                w = pl.get_size()[0] / Pango.SCALE
                if w > max:
                    max = w
            return max
        else:
            return self.rect.width

    def label_safe_width(self):
        ''' Return maximum width for a label '''
        return self.rect.width - self._margins[0] - self._margins[2]

    def label_safe_height(self):
        ''' Return maximum height for a label '''
        return self.rect.height - self._margins[1] - self._margins[3]

    def label_left_top(self):
        ''' Return the upper-left corner of the label safe zone '''
        return(self._margins[0], self._margins[1])

    def get_pixel(self, pos, i=0):
        ''' Return the pixel at (x, y) '''
        x = int(pos[0] - self.rect.x)
        y = int(pos[1] - self.rect.y)
        if x < 0 or x > (self.rect.width - 1) or \
                y < 0 or y > (self.rect.height - 1):
            return(-1, -1, -1, -1)
        # Create a new 1x1 cairo surface.
        cs = cairo.ImageSurface(cairo.FORMAT_RGB24, 1, 1)
        cr = cairo.Context(cs)
        cr.set_source_surface(self.cached_surfaces[i], -x, -y)
        cr.rectangle(0, 0, 1, 1)
        cr.set_operator(cairo.OPERATOR_SOURCE)
        cr.fill()
        cs.flush()  # Ensure all the writing is done.
        pixels = cs.get_data()  # Read the pixel.
        return (pixels[2], pixels[1], pixels[0], 0)



# A Turtle Block based on the Speak Activity interface to AIML
# Copyright 2012 Walter Bender, Sugar Labs
#
# Copyright (C) 2008 Sebastian Silva Fundacion FuenteLibre
# sebastian@fuentelibre.org
#
# Style and structure taken from Speak.activity Copyright (C) Joshua Minor
#
#     HablarConSara.activity is free software: you can redistribute it
#     and/or modify it under the terms of the GNU General Public
#     License as published by the Free Software Foundation, either
#     version 3 of the License, or (at your option) any later version.
#
#     HablarConSara.activity is distributed in the hope that it will be useful,
#     but WITHOUT ANY WARRANTY; without even the implied warranty of
#     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#     GNU General Public License for more details.
#
#     You should have received a copy of the GNU General Public
#     License along with HablarConSara.activity.  If not, see
#     <http://www.gnu.org/licenses/>.


def myblock(tw, args):
    ''' Dialog with AIML library: Usage: Load this code into a Python
        Block. Pass text as an argument and the robot's response will
        be pushed to the stack. Use a Pop Block to pop the response
        off the the stack.'''

    # The AIML library is bundled with the Speak activity
    SPEAKPATHS = ['/home/olpc/Activities/Speak.activity',
                  '/home/liveuser/Activities/Speak.activity',
                  '/usr/share/sugar/activities/Speak.activity']
    import os
    from gettext import gettext as _
    speakpath = None
    for sp in SPEAKPATHS:
        if os.path.exists(sp):
            speakpath = sp
            break
    if speakpath is None:
        tw.showlabel(
            'status', _('Please install the Speak Activity and try again.'))
        return
    import sys
    sys.path.append(speakpath)

    import aiml
    import voice

    BOTS = {
        _('Spanish'): {'name': 'Sara',
                       'brain': os.path.join(speakpath, 'bot', 'sara.brn'),
                       'predicates': {'nombre_bot': 'Sara',
                                      'botmaster': 'La comunidad Azucar'}},
        _('English'): {'name': 'Alice',
                       'brain': os.path.join(speakpath, 'bot', 'alice.brn'),
                       'predicates': {'name': 'Alice',
                                      'master': 'The Sugar Community'}}}

    def get_mem_info(tag):
        meminfo = file('/proc/meminfo').readlines()
        return int([i for i in meminfo if i.startswith(tag)][0].split()[1])

    # load Standard AIML set for restricted systems
    if get_mem_info('MemTotal:') < 524288:
        mem_free = get_mem_info('MemFree:') + get_mem_info('Cached:')
        if mem_free < 102400:
            BOTS[_('English')]['brain'] = None
        else:
            BOTS[_('English')]['brain'] = os.path.join(speakpath, 'bot',
                                                       'alisochka.brn')

    def get_default_voice():
        default_voice = voice.defaultVoice()
        if default_voice.friendlyname not in BOTS:
            return voice.allVoices()[_('English')]
        else:
            return default_voice

    def brain_respond(kernel, text):
        if kernel is not None:
            text = kernel.respond(text)
        if kernel is None or not text:
            text = ''
            tw.showlabel(
                'status',
                _("Sorry, I can't understand what you are asking about."))
        return text

    def brain_load(kernel, voice):
        brain = BOTS[voice.friendlyname]
        kernel = aiml.Kernel()

        if brain['brain'] is None:
            tw.showlabel(
                'status', _('Sorry, there is no free memory to load my brain. \
Close other activities and try once more.'))
            return kernel

        kernel.loadBrain(brain['brain'])
        for name, value in brain['predicates'].items():
            kernel.setBotPredicate(name, value)

        return kernel

    text = args[0]
    if not hasattr(tw, 'aiml_kernel'):
        tw.aiml_kernel = brain_load(tw, get_default_voice())
    response_text = brain_respond(tw.aiml_kernel, text)
    tw.lc.heap.append(response_text)
    return



# Copyright (c) 2009-10, Walter Bender (on behalf of Sugar Labs)

# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:

# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.

# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.

# A naive approach to running myfunc in a jail
import traceback


def myfunc(f, args):
    ''' Run inline Python code '''
    # check to make sure no import calls are made
    params = ", ".join(['x', 'y', 'z'][:len(args)])
    myf = ''.join(['def f(', params, '): return ', f.replace('import', '')])
    userdefined = {}
    exec(myf, globals(), userdefined)
    return list(userdefined.values())[0](*args)


def myfunc_import(parent, f, args):
    ''' Run Python code imported from Journal '''
    if 'def myblock(lc,' in f:
        base_class = parent.tw.lc  # pre-v107, we passed lc
    else:
        base_class = parent.tw  # as of v107, we pass tw
    userdefined = {}
    try:
        exec(f, globals(), userdefined)
        return userdefined['myblock'](base_class, args)
    except BaseException:
        traceback.print_exc()
        return None



#!/usr/bin/env python
# Copyright (c) 2011, 2012 Walter Bender
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA


from gi.repository import Gst
Gst.init(None)
from fcntl import ioctl
import os
from time import time

from gettext import gettext as _

from plugins.camera_sensor.tacamera import Camera
from plugins.camera_sensor.v4l2 import v4l2_control, V4L2_CID_AUTOGAIN, \
    VIDIOC_G_CTRL, VIDIOC_S_CTRL

from plugins.plugin import Plugin

from TurtleArt.tapalette import make_palette
from TurtleArt.talogo import media_blocks_dictionary
from TurtleArt.tautils import get_path, debug_output, power_manager_off
from TurtleArt.taconstants import MEDIA_SHAPES, NO_IMPORT, SKIN_PATHS, \
    BLOCKS_WITH_SKIN
from TurtleArt.taprimitive import (ConstantArg, Primitive)
from TurtleArt.tatype import TYPE_NUMBER


class Camera_sensor(Plugin):

    def __init__(self, parent):
        Plugin.__init__(self)
        ''' Make sure there is a camera device '''
        self._parent = parent
        self._status = False
        self._ag_control = None
        self.devices = []
        self.cameras = []
        self.luminance = 0

        if os.path.exists('/dev/video0'):
            self.devices.append('/dev/video0')
        if os.path.exists('/dev/video1'):
            self.devices.append('/dev/video1')
        if len(self.devices) > 0:
            self._status = True
        else:
            self._status = False

    def setup(self):
        ''' Set up the palettes '''
        sensors_palette = make_palette('sensor',
                                       colors=["#FF6060", "#A06060"],
                                       help_string=_(
                                           'Palette of sensor blocks'),
                                       position=6)
        media_palette = make_palette('media',
                                     colors=["#A0FF00", "#80A000"],
                                     help_string=_('Palette of media objects'),
                                     position=7)

        # set up camera-specific blocks
        media_blocks_dictionary['camera'] = self.prim_take_picture0
        media_blocks_dictionary['camera1'] = self.prim_take_picture1

        SKIN_PATHS.append('plugins/camera_sensor/images')

        hidden = True
        second_cam = False
        if self._status:
            hidden = False
            if len(self.devices) > 1:
                second_cam = True

        sensors_palette.add_block('luminance',
                                  hidden=hidden,
                                  style='box-style',
                                  label=_('brightness'),
                                  help_string=_(
                                      'light level detected by camera'),
                                  value_block=True,
                                  prim_name='luminance')
        self._parent.lc.def_prim(
            'luminance', 0,
            Primitive(self.prim_read_camera,
                      return_type=TYPE_NUMBER,
                      kwarg_descs={'luminance_only': ConstantArg(True)},
                      call_afterwards=self.after_luminance))

        media_palette.add_block('camera',
                                hidden=hidden,
                                style='box-style-media',
                                label=' ',
                                default='CAMERA',
                                help_string=_('camera output'),
                                content_block=True)

        media_palette.add_block('camera1',
                                hidden=not(second_cam),
                                style='box-style-media',
                                label=' ',
                                default='CAMERA',
                                help_string=_('camera output'),
                                content_block=True)

        # Depreciated block
        sensors_palette.add_block(
            'read_camera',
            hidden=True,
            style='box-style',
            label=_('brightness'),
            help_string=_(
                'Average RGB color from camera is pushed to the stack'),
            value_block=True,
            prim_name='read_camera')
        self._parent.lc.def_prim(
            'read_camera', 0,
            Primitive(self.prim_read_camera,
                      return_type=TYPE_NUMBER,
                      kwarg_descs={'luminance_only': ConstantArg(False)}))

        NO_IMPORT.append('camera')
        BLOCKS_WITH_SKIN.append('camera')
        NO_IMPORT.append('camera1')
        BLOCKS_WITH_SKIN.append('camera1')
        MEDIA_SHAPES.append('camerasmall')
        MEDIA_SHAPES.append('cameraoff')
        MEDIA_SHAPES.append('camera1small')
        MEDIA_SHAPES.append('camera1off')

    def start(self):
        ''' Initialize the camera if there is an camera block in use '''
        camera_blocks = len(self._parent.block_list.get_similar_blocks(
            'block', ['camera', 'camera1', 'read_camera', 'luminance']))
        if not self._parent.running_turtleart or camera_blocks > 0:
            if self._status and len(self.cameras) == 0:
                for device in self.devices:
                    self.cameras.append(Camera(device))
                power_manager_off(True)

    def quit(self):
        ''' This gets called when the activity quits '''
        self._reset_the_camera()

    def stop(self):
        ''' This gets called by the stop button '''
        self._reset_the_camera()

    def clear(self):
        ''' This gets called by the clean button and erase button '''
        self._reset_the_camera()

    def _reset_the_camera(self):
        if self._status and len(self.cameras) > 0:
            for i, camera in enumerate(self.cameras):
                camera.stop_camera_input()
                self._set_autogain(1, camera=i)  # enable AUTOGAIN
            power_manager_off(False)

    def _status_report(self):
        debug_output('Reporting camera status: %s' % (str(self._status)),
                     self._parent.running_sugar)
        return self._status

    # Block primitives used in talogo

    def prim_take_picture0(self):
        self._take_picture(camera=0)

    def prim_take_picture1(self):
        self._take_picture(camera=1)

    def _take_picture(self, camera=0):
        ''' method called by media block '''
        self._set_autogain(1, camera)  # enable AUTOGAIN
        self._get_pixbuf_from_camera(camera)
        self._parent.lc.pixbuf = self.cameras[camera].pixbuf

    def prim_read_camera(self, luminance_only=False, camera=0):
        """ Read average pixel from camera and push b, g, r to the stack """
        self.luminance_only = luminance_only
        if not self._status:
            if self.luminance_only:
                return -1
            else:
                self._parent.lc.heap.append(-1)
                self._parent.lc.heap.append(-1)
                self._parent.lc.heap.append(-1)
                return
        self._set_autogain(0, camera=camera)  # disable AUTOGAIN
        self._get_pixbuf_from_camera(camera=camera)
        self.calc_luminance(camera=camera)
        if self.luminance_only:
            return int(self.luminance)
        else:
            self._parent.lc.heap.append(self.b)
            self._parent.lc.heap.append(self.g)
            self._parent.lc.heap.append(self.r)
            return

    def calc_luminance(self, camera=0):
        array = self.cameras[camera].pixbuf.get_pixels()
        width = self.cameras[camera].pixbuf.get_width()
        height = self.cameras[camera].pixbuf.get_height()

        if array is not None:
            length = int(len(array) / 3)
            if length != width * height:
                debug_output('array length != width x height (%d != %dx%d)' %
                             (length, width, height),
                             self._parent.running_sugar)

            # Average the 100 pixels in the center of the screen
            r, g, b = 0, 0, 0
            row_offset = int((height / 2 - 5) * width * 3)
            column_offset = int(width / 2 - 5) * 3
            for y in range(10):
                i = row_offset + column_offset
                for x in range(10):
                    r += ord(array[i])
                    i += 1
                    g += ord(array[i])
                    i += 1
                    b += ord(array[i])
                    i += 1
                row_offset += width * 3
            if self.luminance_only:
                self.luminance = int((r * 0.3 + g * 0.6 + b * 0.1) / 100)
            else:
                self.r = int(r / 100)
                self.g = int(g / 100)
                self.b = int(b / 100)
        else:
            if self.luminance_only:
                self.luminance = -1
            else:
                self.r = -1
                self.g = -1
                self.b = -1

    def after_luminance(self, luminance_only=False):
        if self._parent.lc.update_values and luminance_only:
            self._parent.lc.update_label_value('luminance', self.luminance)

    def _set_autogain(self, state, camera=0):
        ''' 0 is off; 1 is on '''
        if self._ag_control is not None and self._ag_control.value == state:
            return
        try:
            video_capture_device = open(self.devices[camera], 'rw')
        except BaseException:
            video_capture_device = None
            debug_output('video capture device not available',
                         self._parent.running_sugar)
            return
        self._ag_control = v4l2_control(V4L2_CID_AUTOGAIN)
        try:
            ioctl(video_capture_device, VIDIOC_G_CTRL, self._ag_control)
            self._ag_control.value = state
            ioctl(video_capture_device, VIDIOC_S_CTRL, self._ag_control)
        except BaseException:
            pass
        video_capture_device.close()

    def _get_pixbuf_from_camera(self, camera):
        ''' Regardless of how we get it, we want to return a pixbuf '''
        self._parent.lc.pixbuf = None
        if self._status:
            self.cameras[camera].start_camera_input()



#!/usr/bin/env python
# Copyright (c) 2011 Walter Bender
# Copyright (c) 2011 Collabora Ltd. <http://www.collabora.co.uk/>

# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:

# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.

# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.

import sys
sys.path.append("..")
import os.path

import dbus
from gettext import gettext as _

from plugin import Plugin

from TurtleArt.util.menubuilder import make_menu_item, make_sub_menu
from TurtleArt.util.configfile import ConfigFile
from TurtleArt.util.configwizard import ConfigWizard

from collaboration.neighborhood import get_neighborhood
from collaboration.connectionmanager import get_connection_manager
from collaboration.activity import Activity
from collaboration import telepathyclient

from TurtleArt.tacollaboration import Collaboration

import traceback

from gi.repository import Gtk
from gi.repository import GObject

CONNECTION_INTERFACE_ACTIVITY_PROPERTIES = \
    'org.laptop.Telepathy.ActivityProperties'


class Collaboration_plugin(Plugin):

    __gsignals__ = {
        'joined': (GObject.SIGNAL_RUN_FIRST, GObject.TYPE_NONE,
                   ()),
        'shared': (GObject.SIGNAL_RUN_FIRST, GObject.TYPE_NONE,
                   ()), }

    def __init__(self, parent):
        Plugin.__init__(self)

        self._parent = parent
        self._neighborhood = None
        self._title = _('My Turtle Art session')
        self._bundle_id = "org.laptop.TurtleArt"
        # This could be hashed from the file path (if resuming)
        self._activity_id = "1234567"
        self._nick = ""
        self._setup_has_been_called = False

    def _setup_config_file(self, config_file_path):
        self._config_file_path = os.path.join(config_file_path,
                                              'turtleartrc.collab')
        self._collaboration_config_values = ConfigFile(self._config_file_path)
        self._valid_config_values = {
            'nick': {'type': 'text'},
            'account_id': {'type': 'text'},
            'password': {'type': 'text'},
            'server': {'type': 'text'},
            'port': {'type': 'integer'},
            'register': {'type': 'boolean'},
            'colors': {'type': 'text'}}

    def _connect_cb(self, button):
        """ Enable connection """
        self._collaboration_config_values.set_valid_keys(
            self._valid_config_values)
        self._collaboration_config_values.connect(
            'configuration-loaded', self._connect_to_neighborhood)
        self._collaboration_config_values.connect(
            'configuration-saved', self._connect_to_neighborhood)
        self._collaboration_config_values.load()
        self.setup()

    def setup(self):
        self._collaboration = Collaboration(self.tw, self)
        self._collaboration.setup()
        # Do we know if we were successful?
        self._setup_has_been_called = True
        # TODO:
        #     use set_sensitive to enable Share and Configuration menuitems

    def set_tw(self, turtleart_window):
        self.tw = turtleart_window
        self.tw.nick = self._get_nick()
        self._setup_config_file(self._parent.get_config_home())

    def get_menu(self):
        menu = Gtk.Menu()

        make_menu_item(menu, _('Enable collaboration'),
                       self._connect_cb)

        self._activities_submenu = Gtk.Menu()
        activities_menu = make_sub_menu(self._activities_submenu,
                                        _('Activities'))
        menu.append(activities_menu)

        self._buddies_submenu = Gtk.Menu()
        buddies_menu = make_sub_menu(self._buddies_submenu,
                                     _('Buddies'))
        menu.append(buddies_menu)

        make_menu_item(menu, _('Share'), self._share_cb)
        make_menu_item(menu, _('Configuration'),
                       self._config_neighborhood_cb)

        neighborhood_menu = make_sub_menu(menu, _('Neighborhood'))

        return neighborhood_menu

    def send_xy(self):
        ''' Resync xy position (and orientation) of my turtle. '''
        self._collaboration.send_my_xy()

    def get_colors(self):
        return self._colors

    def _get_nick(self):
        return self._nick

    def _get_activity_id(self):
        return self._activity_id

    def _get_bundle_id(self):
        return self._bundle_id

    def _get_title(self):
        return self._title

    def _connect_to_neighborhood(self, config_file_obj):
        if self._neighborhood is not None:
            return

        params = {}
        params['nickname'] = self._collaboration_config_values.get('nick')
        params['account_id'] = self._collaboration_config_values.get(
            'account_id')
        params['server'] = self._collaboration_config_values.get('server')
        params['port'] = self._collaboration_config_values.get('port')
        params['password'] = self._collaboration_config_values.get('password')
        params['register'] = self._collaboration_config_values.get('register')
        if params['server'] == '':
            raise RuntimeError('Invalid server address')

        self._nick = self._collaboration_config_values.get('nick')
        # Tell the parent activity that the nick may have changed
        self._parent.nick_changed(self._nick)

        self._colors = self._collaboration_config_values.get('colors')
        # Tell the parent activity that the colors may have changed
        self._parent.color_changed(self._colors)

        self._activities = {}
        self._buddies = {}

        dbus.mainloop.glib.DBusGMainLoop(set_as_default=True)

        self._client_handler = telepathyclient.get_instance()
        if self._client_handler is None:
            raise RuntimeError('Telepathy client unavailable')
        self._neighborhood = get_neighborhood(params)
        self._neighborhood.connect('activity-added', self._activity_added_cb)
        self._neighborhood.connect('activity-removed',
                                   self._activity_removed_cb)
        self._neighborhood.connect('buddy-added', self._buddy_added_cb)
        self._neighborhood.connect('buddy-removed', self._buddy_removed_cb)

    # TODO:
    # - show nick of sharer
    # - show icon with color of sharer
    def _activity_added_cb(self, model, activity_model):
        self._activities[activity_model.props.name] = activity_model
        self._recreate_available_activities_menu()

    def _activity_removed_cb(self, model, activity_model):
        try:
            self._activities.pop(activity_model.props.name)
        except BaseException:
            print 'Failed to remove activity %s' % activity_model.props.name

        self._recreate_available_activities_menu()

    def _buddy_added_cb(self, activity, buddy):
        self._buddies[buddy.get_key()] = buddy
        self._recreate_available_buddies_menu()

    def _buddy_removed_cb(self, activity, buddy):
        try:
            self._buddies.pop(buddy.get_key())
        except BaseException:
            print "Couldn't remove buddy %s" % buddy.get_key()
        self._recreate_available_buddies_menu()

    # TODO: we should have a list of available actions over
    #       a given buddy. I.e.: a) chat with him b) make friend
    #       c) invite to current activity
    #
    def _recreate_available_buddies_menu(self):
        for child in self._buddies_submenu.get_children():
            self._buddies_submenu.remove(child)

        for buddy in self._buddies.values():
            key = buddy.get_key()
            if key is None:
                key = ''
            n = buddy.get_nick() + '|' + key[0:15]
            make_menu_item(self._buddies_submenu, n,
                           self._buddy_actions_cb, buddy)

    def _buddy_actions_cb(self, widget, buddy):
        print 'do something with %s' % buddy.get_nick()

    # TODO:
    #     we need an extra menu branch with a) 'Join' button b) List of buddies
    def _recreate_available_activities_menu(self):
        for child in self._activities_submenu.get_children():
            self._activities_submenu.remove(child)

        for activity in self._activities.values():
            n = activity.props.name
            make_menu_item(self._activities_submenu, n,
                           self._join_activity_cb, activity)

    def _join_activity_cb(self, widget, activity):
        print 'Lets try to join...'

        connection_manager = get_connection_manager()
        account_path, connection = \
            connection_manager.get_preferred_connection()
        if connection is None:
            print('No active connection available')
            return

        properties = {}
        properties['id'] = activity.activity_id
        properties['color'] = activity.get_color()
        print 'room handle according to activity %s' % activity.room_handle
        properties['private'] = True

        try:
            room_handle = connection.GetActivity(
                activity.activity_id,
                dbus_interface=CONNECTION_INTERFACE_ACTIVITY_PROPERTIES)
            print('room_handle = %s' % str(room_handle))
            self._joined_activity = Activity(
                account_path, connection, room_handle, properties=properties)
            # FIXME: this should be unified, no need to keep 2 references
            self._shared_activity = self._joined_activity
        except BaseException:
            traceback.print_exc(file=sys.stdout)

        if self._joined_activity.props.joined:
            raise RuntimeError('Activity %s is already shared.' %
                               activity.activity_id)

        self._joined_activity.connect('joined', self.__joined_cb)
        self._joined_activity.join()

    def __joined_cb(self, activity, success, err):
        print "We've joined an activity"
        self.emit('joined')

    def _config_neighborhood_cb(self, widget):
        if not self._setup_has_been_called:
            return
        config_w = ConfigWizard(self._config_file_path)
        config_items = [
            {'item_label': _('Nickname'), 'item_type': 'text',
             'item_name': 'nick'},
            {'item_label': _('Account ID'), 'item_type': 'text',
             'item_name': 'account_id'},
            {'item_label': _('Server'), 'item_type': 'text',
             'item_name': 'server'},
            {'item_label': _('Port'), 'item_type': 'text',
             'item_name': 'port'},
            {'item_label': _('Password'), 'item_type': 'text',
             'item_name': 'password'},
            {'item_label': _('Register'), 'item_type': 'boolean',
             'item_name': 'register'},
            {'item_label': _('Colors'), 'item_type': 'text',
             'item_name': 'colors'}]
        config_w.set_config_items(config_items)
        config_w.set_config_file_obj(self._collaboration_config_values)
        config_w.show()

    def _share_cb(self, button):
        if not self._setup_has_been_called:
            return
        properties = {}
        properties['id'] = self._get_activity_id()
        properties['type'] = self._get_bundle_id()
        properties['name'] = self._get_title()
        properties['color'] = self.get_colors()
        properties['private'] = False

        connection_manager = get_connection_manager()
        account_path, connection = \
            connection_manager.get_preferred_connection()

        if connection is None:
            print('No active connection available')
            return

        try:
            self._parent._shared_activity = Activity(account_path,
                                                     connection,
                                                     properties=properties)
            # FIXME: this should be unified, no need to keep 2 references
            self._shared_activity = self._parent._shared_activity
        except BaseException:
            traceback.print_exc(file=sys.stdout)

        if self._parent._shared_parent.props.joined:
            raise RuntimeError('Activity %s is already shared.' %
                               self._parent._get_activity_id())

        self._parent._shared_parent.share(self.__share_activity_cb,
                                          self.__share_activity_error_cb)

    def __share_activity_cb(self, activity):
        """Finish sharing the activity"""
        self.emit('shared')

    def __share_activity_error_cb(self, activity, error):
        """Notify with GObject event of unsuccessful sharing of activity"""
        print '%s got error: %s' % (activity, error)


if __name__ == '__main__':
    print 'testing collaboration'



import gobject


class RFIDDevice(gobject.GObject):
    """
    Ancestor class for every supported device.
    The main class for the device driver must be called "RFIDReader".
    """
    # signal "tag-read" has to be emitted when a tag has been read.
    # The handler must receive the ISO-11784 hex value of the tag.
    # signal "disconnected" has to be emitted when the device is
    # unplugged or an error has been detected.
    __gsignals__ = {
        'tag-read': (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE,
                     (gobject.TYPE_STRING,)),
        'disconnected': (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE,
                         (gobject.TYPE_STRING,))
    }

    def __init__(self):
        """
        Initializer. Subclasses must call this method.
        """
        self.__gobject_init__()

    def get_present(self):
        """
        This method must detect if the device is present, returning True if so,
        or False otherwise.
        """
        raise NotImplementedError

    def get_version(self):
        """
        Returns a descriptive text of the device.
        """
        raise NotImplementedError

    def do_connect(self):
        """
        Connects to the device.
        Must return True if successfull, False otherwise.
        """
        raise NotImplementedError

    def do_disconnect(self):
        """
        Disconnects from the device.
        """
        raise NotImplementedError

    def read_tag(self):
        """
        Returns the 64 bit data in hex format of the last read tag.
        """
        raise NotImplementedError

    def write_tag(self, hex_val):
        """
        Could be implemented if the device is capable of writing tags.
        Receives the hex value (according to ISO 11784) to be written.
        Returns True if successfull or False if something went wrong.
        """



# -*- coding: utf-8 -*-
# Copyright (c) 2010-14 Walter Bender

# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:

# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.

# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.

import os
import tempfile
from random import uniform
from gettext import gettext as _

# Packaging constants
SUFFIX = ('.ta', '.tb')
MAGICNUMBER = 'TB'
MIMETYPE = ['application/x-turtle-art', 'application/vnd.turtleblocks']

# Sprite layers
OVERLAY_LAYER = 400
TURTLE_LAYER = 500
BLOCK_LAYER = 600
CATEGORY_LAYER = 700
TAB_LAYER = 800
PROTO_LAYER = 801
STATUS_LAYER = 900
TOP_LAYER = 1000

# Special-case some block colors
BOX_COLORS = {'red': ["#FF0000", "#A00000"],
              'orange': ["#FFD000", "#AA8000"],
              'yellow': ["#FFFF00", "#A0A000"],
              'blue': ["#0000FF", "#000080"],
              'cyan': ["#00FFFF", "#00A0A0"],
              'green': ["#00FF00", "#008000"],
              'purple': ["#FF00FF", "#A000A0"],
              'white': ["#FFFFFF", "#A0A0A0"],
              'black': ["#000000", "#000000"]}

# Misc. parameters
PALETTE_HEIGHT = 120
PALETTE_WIDTH = 175
SELECTOR_WIDTH = 55
ICON_SIZE = 55
GRADIENT_COLOR = "#FFFFFF"
STANDARD_STROKE_WIDTH = 1.0
BLOCK_SCALE = [0.5, 1.0, 1.5, 2.0, 3.0, 4.0, 6.0, 8.0]
PALETTE_SCALE = 1.5
DEFAULT_TURTLE = 'Yertle'
DEFAULT_TURTLE_COLORS = ['#008000', '#00A000']
PALETTE_COLOR = "#FFD000"
TOOLBAR_COLOR = "#282828"
HORIZONTAL_PALETTE = 0
VERTICAL_PALETTE = 1
BLACK = -9999
WHITE = -9998
HIT_HIDE = 240
HIT_SHOW = 224
HIT_RED = "#F00000"
HIT_GREEN = "#00E000"
HIDE_WHITE = "#F0F0F0"
SHOW_WHITE = "#E0E0E0"
DEFAULT_SCALE = 33
DEFAULT_PEN_COLOR = [255, 0, 0]
DEFAULT_BACKGROUND_COLOR = [153, 204, 255]
DEFAULT_BACKGROUND_COLOR_SHADE_GRAY = [60, 80, 100]
DEFAULT_FONT = 'Sans'
XO1 = 'xo1'
XO15 = 'xo1.5'
XO175 = 'xo1.75'
XO30 = 'xo3.0'
XO4 = 'xo4'
UNKNOWN = 'unknown'
PASTE_OFFSET = 20
TMP_SVG_PATH = os.path.join(
    tempfile.gettempdir(), 'turtle-blocks-%d.svg' % int(uniform(0, 10000)))
TMP_ODP_PATH = os.path.join(
    tempfile.gettempdir(), 'turtle-blocks-%d.odp' % int(uniform(0, 10000)))

ARG_MUST_BE_NUMBER = ['product2', 'minus2', 'random', 'remainder2', 'forward',
                      'back', 'left', 'right', 'arc', 'setxy2', 'setxy',
                      'fillscreen', 'setscale', 'setpensize', 'wait',
                      'setcolor', 'seth', 'setgray', 'setshade', 'string',
                      'fillscreen2']

KEY_DICT = {
    'Left': 1,
    'KP_Left': 1,
    'Up': 2,
    'KP_Up': 2,
    'Right': 3,
    'KP_Right': 3,
    'Down': 4,
    'KP_Down': 4,
    'BackSpace': 8,
    'Tab': 9,
    'Return': 13,
    'Escape': 27,
    'space': 32,
    ' ': 32,
    'exclam': 33,
    'quotedbl': 34,
    'numbersign': 35,
    'dollar': 36,
    'percent': 37,
    'ampersand': 38,
    'apostrophe': 39,
    'parenleft': 40,
    'parenright': 41,
    'asterisk': 42,
    'plus': 43,
    'comma': 44,
    'minus': 45,
    'period': 46,
    'slash': 47,
    'colon': 58,
    'semicolon': 59,
    'less': 60,
    'equal': 61,
    'greater': 62,
    'question': 63,
    'at': 64,
    'underscore': 95,
    'bracketleft': 91,
    'backslash': 92,
    'bracketright': 93,
    'asciicircum': 94,
    'grave': 96,
    'braceleft': 123,
    'bar': 124,
    'braceright': 125,
    'asciitilde': 126,
    'Delete': 127,
}
REVERSE_KEY_DICT = {
    1: _('left'),
    2: _('up'),
    3: _('right'),
    4: _('down'),
    8: _('backspace'),
    9: _('tab'),
    # TRANS: enter is the name of the enter (or return) key
    13: _('enter'),
    27: 'esc',
    # TRANS: space is the name of the space key
    32: _('space'),
    127: _('delete')
}


class ColorObj(object):

    def __init__(self, color):
        self.color = color

    def __int__(self):
        if hasattr(self.color, 'color'):
            if self.color.color is None:
                return int(self.color.shade)
            else:
                return int(self.color.color)
        else:
            return int(self.color)

    def __float__(self):
        if hasattr(self.color, 'color'):
            return float(int(self))
        else:
            return float(self.color)

    def __str__(self):
        if isinstance(self.color, (float, int, bool)):
            return str(self.color)
        return str(self.color.name)

    def __repr__(self):
        if isinstance(self.color, (float, int, bool)):
            return str(self.color)
        return str(self.color.name)


class Color(object):

    """ A color used in block programs (e.g., as pen color). """

    def __init__(self, name, color=0, shade=50, gray=100):
        """ name -- a string with the name of the color, e.g., 'red'
        color -- the hue (0-100, or None for white, gray, and black)
        shade -- the lightness (0 is black, 100 is white)
        gray -- the saturation (0 is gray, 100 is fully saturated) """
        self.name = name
        self.color = color
        self.shade = shade
        self.gray = gray

    def __int__(self):
        if self.color is None:
            return int(self.shade)
        else:
            return int(self.color)

    def __float__(self):
        return float(int(self))

    def get_number_string(self):
        return str(int(self))

    def get_number_name(self):
        return str(self.name)

    def __str__(self):
        return str(self.name)

    def __repr__(self):
        return '%s (%s/%d/%d)' % (str(self.name), str(self.color),
                                  self.shade, self.gray)

    def __eq__(self, other):
        """ A Color is equivalent to
        * another Color with the same color, shade, and gray values
        * an integer, float, or long that equals int(self) """
        if isinstance(other, Color):
            return (self.color == other.color and self.shade == other.shade and
                    self.gray == other.gray)
        elif isinstance(other, (int, float)):
            return int(self) == other
        # * a basestring that equals str(self)
        # elif isinstance(other, basestring):
        #    return str(self) == other
        else:
            return False

    def __lt__(self, other):
        """ A Color is less than
        * another Color whose name appears earlier in the alphabet
        * a number that is less than int(self)
        * a string that appears before the underscore in the ASCII table """
        if isinstance(other, Color):
            return str(self) < str(other)
        elif isinstance(other, (int, float)):
            return int(self) < other
        elif isinstance(other, str):
            return '_' + str(self) < other
        else:
            return False

    def __gt__(self, other):
        """ A Color is greater than
        * another Color whose name appears later in the alphabet
        * a number that is greater than int(self)
        * a string that appears after the underscore in the ASCII table """
        if isinstance(other, Color):
            return str(self) > str(other)
        elif isinstance(other, (int, float)):
            return int(self) > other
        elif isinstance(other, str):
            return '_' + str(self) > other
        else:
            return False

    def is_gray(self):
        """ Return True iff this color is white, gray, or black, i.e. if its
        hue is not set or its saturation is zero. """
        return self.color is None or not self.gray


class Vector(object):

    """ A vector object used in block programs (e.g., as food object). """

    def __init__(self, name, vector):
        """ name -- a string with the name of the vector, e.g., 'banana'
        vector -- a list of values, e.g., [105, 1, 27, 3, 0] """
        self.name = name
        self.vector = vector

    '''
    def __int__(self):
        return 0

    def __float__(self):
        return 0.0
    '''

    def __str__(self):
        return self.get_vector_string()

    def get_vector_string(self):
        string = '%s [' % str(self.name)
        for s in self.vector:
            string += '%d' % s
            if self.vector.index(s) < len(self.vector) - 1:
                string += ', '
        string += ']'
        return string

    def __eq__(self, other):
        """ A Vector is equivalent to
        * another Vector with the same vector values """
        if isinstance(other, Vector):
            return self.vector == other.vector
        else:
            return False

    def __lt__(self, other):
        """ A Vector is less than
        * another Vector whose name appears earlier in the alphabet
        * a number that is less than int(self)
        * a string that appears before the underscore in the ASCII table """
        if isinstance(other, Vector):
            return str(self.vector) < str(other.vector)
        else:
            return False

    def __gt__(self, other):
        """ A Vector is greater than
        * another Vector whose name appears later in the alphabet
        * a number that is greater than int(self)
        * a string that appears after the underscore in the ASCII table """
        if isinstance(other, Vector):
            return str(self.vector) > str(other.vector)
        else:
            return False


CONSTANTS = {'leftpos': None, 'toppos': None, 'rightpos': None,
             'bottompos': None, 'width': None, 'height': None,
             'black': Color('black', None, 0, 0),
             'white': Color('white', None, 100, 0),
             'red': Color('red', 0, 50, 100),
             'orange': Color('orange', 10, 50, 100),
             'yellow': Color('yellow', 20, 50, 100),
             'green': Color('green', 40, 50, 100),
             'cyan': Color('cyan', 50, 50, 100),
             'blue': Color('blue', 70, 50, 100),
             'purple': Color('purple', 90, 50, 100),
             'titlex': None, 'titley': None, 'leftx': None,
             'topy': None, 'rightx': None, 'bottomy': None}

# Blocks that are expandable
EXPANDABLE_STYLE = ['boolean-style', 'compare-porch-style', 'compare-style',
                    'number-style-porch', 'number-style', 'basic-style-2arg',
                    'basic-style-3arg', 'number-style-block',
                    'box-style-media']

# These are defined in add_block based on block style
EXPANDABLE_FLOW = []

EXPANDABLE = ['vspace', 'hspace', 'identity2']

EXPANDABLE_ARGS = ['bulletlist', 'myfunc1arg', 'myfunc2arg', 'myfunc3arg',
                   'userdefined', 'userdefined2args', 'userdefined3args',
                   'loadblock', 'loadblock2arg', 'loadblock3arg']

# Deprecated block styles that need dock adjustments
OLD_DOCK = ['and', 'or', 'plus', 'minus', 'division', 'product', 'remainder']

CONTENT_ARGS = ['show', 'showaligned', 'push', 'storein', 'storeinbox1',
                'storeinbox2']

MEDIA_BLOCK2TYPE = {}  # map media blocks to media types


BLOCKS_WITH_SKIN = []  # These blocks get a special skin

PYTHON_SKIN = []

SKIN_PATHS = ['images']

MEDIA_SHAPES = []

NO_IMPORT = []

EXPAND_SKIN = {}

# Status blocks
OVERLAY_SHAPES = ['Cartesian', 'Cartesian_labeled', 'polar', 'metric']

STATUS_SHAPES = ['status', 'info', 'nostack', 'dupstack', 'noinput',
                 'emptyheap', 'emptybox', 'nomedia', 'nocode', 'overflowerror',
                 'negroot', 'syntaxerror', 'nofile', 'nojournal', 'zerodivide',
                 'notanumber', 'incompatible', 'help', 'print', 'noconnection',
                 'emptystart']

# Emulate Sugar toolbar when running from outside of Sugar
TOOLBAR_SHAPES = ['hideshowoff', 'eraseron', 'run-fastoff',
                  'run-slowoff', 'stopiton']

# Legacy names
OLD_NAMES = {'product': 'product2', 'storeinbox': 'storein', 'minus': 'minus2',
             'division': 'division2', 'plus': 'plus2', 'and': 'and2',
             'or': 'or2', 'less': 'less2', 'greater': 'greater2',
             'equal': 'equal2', 'remainder': 'remainder2',
             'identity': 'identity2', 'division': 'division2',
             'audiooff': 'audio', 'endfill': 'stopfill',
             'descriptionoff': 'description', 'template3': 'templatelist',
             'template1': 'template1x1', 'template2': 'template2x1',
             'template6': 'template1x2', 'template7': 'template2x2',
             'template4': 'template1x1a', 'hres': 'width', 'vres': 'height',
             'sandwichtop2': 'sandwichtop', 'image': 'show',
             'container': 'indentity2', 'insertimage': 'show'}

# Define the relative size and postion of media objects
#                    (w, h, x, y, dx, dy)
#
TITLEXY = (0.9375, 0.875)

# Relative placement of portfolio objects (used by deprecated blocks)
TEMPLATES = {'t1x1': (0.5, 0.5, 0.0625, 0.125, 1.05, 0),
             't2z1': (0.5, 0.5, 0.0625, 0.125, 1.05, 1.05),
             't1x2': (0.45, 0.45, 0.0625, 0.125, 1.05, 1.05),
             't2x2': (0.45, 0.45, 0.0625, 0.125, 1.05, 1.05),
             't1x1a': (0.9, 0.9, 0.0625, 0.125, 0, 0),
             'bullet': (1, 1, 0.0625, 0.125, 0, 0.1),
             'insertimage': (0.333, 0.333)}

RETURN = '⏎'

VOICES = {'af': 'afrikaans', 'cy': 'welsh-test', 'el': 'greek',
          'es': 'spanish', 'hi': 'hindi-test', 'hy': 'armenian',
          'ku': 'kurdish', 'mk': 'macedonian-test', 'pt': 'brazil',
          'sk': 'slovak', 'sw': 'swahili', 'bs': 'bosnian',
          'da': 'danish', 'en': 'english', 'fi': 'finnish',
          'hr': 'croatian', 'id': 'indonesian-test', 'la': 'latin',
          'nl': 'dutch-test', 'sq': 'albanian', 'ta': 'tamil',
          'vi': 'vietnam-test', 'ca': 'catalan', 'de': 'german',
          'eo': 'esperanto', 'fr': 'french', 'hu': 'hungarian',
          'is': 'icelandic-test', 'lv': 'latvian', 'no': 'norwegian',
          'ro': 'romanian', 'sr': 'serbian', 'zh': 'Mandarin',
          'cs': 'czech', 'it': 'italian', 'pl': 'polish',
          'ru': 'russian_test', 'sv': 'swedish', 'tr': 'turkish'}

# Macros (groups of blocks)
MACROS = {
    'ifthenelse':  # Because it is too big to fit on the palette
    [[0, 'ifelse', 0, 0, [None, None, None, None, None]]],
    'indexblock':
    [[0, 'index', 0, 0, [None, 1, 2, 3]],
     [1, ['string', _('text')], 0, 0, [0, None]],
     [2, ['number', 0], 0, 0, [0, None]],
     [3, ['number', 1], 0, 0, [0, None]]],
    'kbinput':
    [[0, 'until', 0, 0, [None, 1, 4, None]],
     [1, 'greater2', 0, 0, [0, 2, 3, None]],
     [2, 'keyboard', 0, 0, [1, None]],
     [3, ['number', '0'], 0, 0, [1, None]],
     [4, 'wait', 0, 0, [0, 5, 6]],
     [5, ['number', '0.1'], 0, 0, [4, None]],
     [6, 'kbinput', 0, 0, [4, None]]],
    'list':
    [[0, ['sandwichclamp', 252], 0, 0, [None, 1, None]],
     [1, 'penup', 0, 0, [0, 2]],
     [2, 'setxy2', 0, 0, [1, 3, 4, 5]],
     [3, 'titlex', 0, 0, [2, None]],
     [4, 'titley', 0, 0, [2, None]],
     [5, 'pendown', 0, 0, [2, 6]],
     [6, 'setscale', 0, 0, [5, 7, 8]],
     [7, ['number', '100'], 0, 0, [6, None]],
     [8, 'show', 0, 0, [6, 9, 10]],
     [9, ['string', _('Title')], 0, 0, [8, None]],
     [10, 'penup', 0, 0, [8, 11]],
     [11, 'setxy2', 0, 0, [10, 12, 13, 14]],
     [12, 'leftx', 0, 0, [11, None]],
     [13, 'topy', 0, 0, [11, None]],
     [14, 'pendown', 0, 0, [11, 15]],
     [15, 'setscale', 0, 0, [14, 16, 17]],
     [16, ['number', '67'], 0, 0, [15, None]],
     [17, 'bulletlist', 0, 0, [15, 18, 19, None]],
     [18, ['string', '∙ '], 0, 0, [17, None]],
     [19, ['string', '∙ '], 0, 0, [17, None]]],
    '1x1a':
    [[0, ['sandwichclamp', 231], 0, 0, [None, 1, None]],
     [1, 'penup', 0, 0, [0, 2]],
     [2, 'setxy2', 0, 0, [1, 3, 4, 5]],
     [3, 'titlex', 0, 0, [2, None]],
     [4, 'titley', 0, 0, [2, None]],
     [5, 'pendown', 0, 0, [2, 6]],
     [6, 'setscale', 0, 0, [5, 7, 8]],
     [7, ['number', '100'], 0, 0, [6, None]],
     [8, 'show', 0, 0, [6, 9, 10]],
     [9, ['string', _('Title')], 0, 0, [8, None]],
     [10, 'penup', 0, 0, [8, 11]],
     [11, 'setxy2', 0, 0, [10, 12, 13, 14]],
     [12, 'leftx', 0, 0, [11, None]],
     [13, 'topy', 0, 0, [11, None]],
     [14, 'pendown', 0, 0, [11, 15]],
     [15, 'setscale', 0, 0, [14, 16, 17]],
     [16, ['number', '90'], 0, 0, [15, None]],
     [17, 'showaligned', 0, 0, [15, 18, None]],
     [18, 'journal', 0, 0, [17, None]]],
    '2x2':
    [[0, ['sandwichclamp', 546], 0, 0, [None, 1, None]],
     [1, 'penup', 0, 0, [0, 2]],
     [2, 'setxy2', 0, 0, [1, 3, 4, 5]],
     [3, 'titlex', 0, 0, [2, None]],
     [4, 'titley', 0, 0, [2, None]],
     [5, 'pendown', 0, 0, [2, 6]],
     [6, 'setscale', 0, 0, [5, 7, 8]],
     [7, ['number', '100'], 0, 0, [6, None]],
     [8, 'show', 0, 0, [6, 9, 10]],
     [9, ['string', _('Title')], 0, 0, [8, None]],
     [10, 'setscale', 0, 0, [8, 11, 12]],
     [11, ['number', '35'], 0, 0, [10, None]],
     [12, 'penup', 0, 0, [10, 13]],
     [13, 'setxy2', 0, 0, [12, 14, 15, 16]],
     [14, 'leftx', 0, 0, [13, None]],
     [15, 'topy', 0, 0, [13, None]],
     [16, 'pendown', 0, 0, [13, 17]],
     [17, 'showaligned', 0, 0, [16, 18, 19]],
     [18, 'journal', 0, 0, [17, None]],
     [19, 'penup', 0, 0, [17, 20]],
     [20, 'setxy2', 0, 0, [19, 21, 22, 23]],
     [21, 'rightx', 0, 0, [20, None]],
     [22, 'topy', 0, 0, [20, None]],
     [23, 'pendown', 0, 0, [20, 24]],
     [24, 'showaligned', 0, 0, [23, 25, 26]],
     [25, 'journal', 0, 0, [24, None]],
     [26, 'penup', 0, 0, [24, 27]],
     [27, 'setxy2', 0, 0, [26, 28, 29, 30]],
     [28, 'leftx', 0, 0, [27, None]],
     [29, 'bottomy', 0, 0, [27, None]],
     [30, 'pendown', 0, 0, [27, 31]],
     [31, 'showaligned', 0, 0, [30, 32, 33]],
     [32, 'journal', 0, 0, [31, None]],
     [33, 'penup', 0, 0, [31, 34]],
     [34, 'setxy2', 0, 0, [33, 35, 36, 37]],
     [35, 'rightx', 0, 0, [34, None]],
     [36, 'bottomy', 0, 0, [34, None]],
     [37, 'pendown', 0, 0, [34, 38]],
     [38, 'showaligned', 0, 0, [37, 39, None]],
     [39, 'journal', 0, 0, [38, None]]],
    '1x2':
    [[0, ['sandwichclamp', 546], 0, 0, [None, 1, None]],
     [1, 'penup', 0, 0, [0, 2]],
     [2, 'setxy2', 0, 0, [1, 3, 4, 5]],
     [3, 'titlex', 0, 0, [2, None]],
     [4, 'titley', 0, 0, [2, None]],
     [5, 'pendown', 0, 0, [2, 6]],
     [6, 'setscale', 0, 0, [5, 7, 8]],
     [7, ['number', '100'], 0, 0, [6, None]],
     [8, 'show', 0, 0, [6, 9, 10]],
     [9, ['string', _('Title')], 0, 0, [8, None]],
     [10, 'setscale', 0, 0, [8, 11, 12]],
     [11, ['number', '35'], 0, 0, [10, None]],
     [12, 'penup', 0, 0, [10, 13]],
     [13, 'setxy2', 0, 0, [12, 14, 15, 16]],
     [14, 'leftx', 0, 0, [13, None]],
     [15, 'topy', 0, 0, [13, None]],
     [16, 'pendown', 0, 0, [13, 17]],
     [17, 'showaligned', 0, 0, [16, 18, 19]],
     [18, 'journal', 0, 0, [17, None]],
     [19, 'penup', 0, 0, [17, 20]],
     [20, 'setxy2', 0, 0, [19, 21, 22, 23]],
     [21, 'rightx', 0, 0, [20, None]],
     [22, 'topy', 0, 0, [20, None]],
     [23, 'pendown', 0, 0, [20, 24]],
     [24, 'showaligned', 0, 0, [23, 25, 26]],
     [25, 'description', 0, 0, [24, None]],
     [26, 'penup', 0, 0, [24, 27]],
     [27, 'setxy2', 0, 0, [26, 28, 29, 30]],
     [28, 'leftx', 0, 0, [27, None]],
     [29, 'bottomy', 0, 0, [27, None]],
     [30, 'pendown', 0, 0, [27, 31]],
     [31, 'showaligned', 0, 0, [30, 32, 33]],
     [32, 'journal', 0, 0, [31, None]],
     [33, 'penup', 0, 0, [31, 34]],
     [34, 'setxy2', 0, 0, [33, 35, 36, 37]],
     [35, 'rightx', 0, 0, [34, None]],
     [36, 'bottomy', 0, 0, [34, None]],
     [37, 'pendown', 0, 0, [34, 38]],
     [38, 'showaligned', 0, 0, [37, 39, None]],
     [39, 'description', 0, 0, [38, None]]],
    '2x1':
    [[0, ['sandwichclamp', 546], 0, 0, [None, 1, None]],
     [1, 'penup', 0, 0, [0, 2]],
     [2, 'setxy2', 0, 0, [1, 3, 4, 5]],
     [3, 'titlex', 0, 0, [2, None]],
     [4, 'titley', 0, 0, [2, None]],
     [5, 'pendown', 0, 0, [2, 6]],
     [6, 'setscale', 0, 0, [5, 7, 8]],
     [7, ['number', '100'], 0, 0, [6, None]],
     [8, 'show', 0, 0, [6, 9, 10]],
     [9, ['string', _('Title')], 0, 0, [8, None]],
     [10, 'setscale', 0, 0, [8, 11, 12]],
     [11, ['number', '35'], 0, 0, [10, None]],
     [12, 'penup', 0, 0, [10, 13]],
     [13, 'setxy2', 0, 0, [12, 14, 15, 16]],
     [14, 'leftx', 0, 0, [13, None]],
     [15, 'topy', 0, 0, [13, None]],
     [16, 'pendown', 0, 0, [13, 17]],
     [17, 'showaligned', 0, 0, [16, 18, 19]],
     [18, 'journal', 0, 0, [17, None]],
     [19, 'penup', 0, 0, [17, 20]],
     [20, 'setxy2', 0, 0, [19, 21, 22, 23]],
     [21, 'rightx', 0, 0, [20, None]],
     [22, 'topy', 0, 0, [20, None]],
     [23, 'pendown', 0, 0, [20, 24]],
     [24, 'showaligned', 0, 0, [23, 25, 26]],
     [25, 'journal', 0, 0, [24, None]],
     [26, 'penup', 0, 0, [24, 27]],
     [27, 'setxy2', 0, 0, [26, 28, 29, 30]],
     [28, 'leftx', 0, 0, [27, None]],
     [29, 'bottomy', 0, 0, [27, None]],
     [30, 'pendown', 0, 0, [27, 31]],
     [31, 'showaligned', 0, 0, [30, 32, 33]],
     [32, 'description', 0, 0, [31, None]],
     [33, 'penup', 0, 0, [31, 34]],
     [34, 'setxy2', 0, 0, [33, 35, 36, 37]],
     [35, 'rightx', 0, 0, [34, None]],
     [36, 'bottomy', 0, 0, [34, None]],
     [37, 'pendown', 0, 0, [34, 38]],
     [38, 'showaligned', 0, 0, [37, 39, None]],
     [39, 'description', 0, 0, [38, None]]],
    '1x1':
    [[0, ['sandwichclamp', 336], 0, 0, [None, 1, None]],
     [1, 'penup', 0, 0, [0, 2]],
     [2, 'setxy2', 0, 0, [1, 3, 4, 5]],
     [3, 'titlex', 0, 0, [2, None]],
     [4, 'titley', 0, 0, [2, None]],
     [5, 'pendown', 0, 0, [2, 6]],
     [6, 'setscale', 0, 0, [5, 7, 8]],
     [7, ['number', '100'], 0, 0, [6, None]],
     [8, 'show', 0, 0, [6, 9, 10]],
     [9, ['string', _('Title')], 0, 0, [8, None]],
     [10, 'setscale', 0, 0, [8, 11, 12]],
     [11, ['number', '35'], 0, 0, [10, None]],
     [12, 'penup', 0, 0, [10, 13]],
     [13, 'setxy2', 0, 0, [12, 14, 15, 16]],
     [14, 'leftx', 0, 0, [13, None]],
     [15, 'topy', 0, 0, [13, None]],
     [16, 'pendown', 0, 0, [13, 17]],
     [17, 'showaligned', 0, 0, [16, 18, 19]],
     [18, 'journal', 0, 0, [17, None]],
     [19, 'penup', 0, 0, [17, 20]],
     [20, 'setxy2', 0, 0, [19, 21, 22, 23]],
     [21, 'rightx', 0, 0, [20, None]],
     [22, 'topy', 0, 0, [20, None]],
     [23, 'pendown', 0, 0, [20, 24]],
     [24, 'showaligned', 0, 0, [23, 25, None]],
     [25, 'description', 0, 0, [24, None]]],
    'reskin':
    [[0, 'skin', 0, 0, [None, 1, None]],
     [1, 'journal', 0, 0, [0, None]]],
    'newfood':
    [[0, 'add_food', 0, 0, [None, 1, 2, 3, 4, 5, 6, 7, None]],
     [1, ['string', _('banana')], 0, 0, [0, None]],
     [2, 'journal', 0, 0, [0, None]],
     [3, ['number', '105'], 0, 0, [0, None]],
     [4, ['number', '1'], 0, 0, [0, None]],
     [5, ['number', '27'], 0, 0, [0, None]],
     [6, ['number', '3'], 0, 0, [0, None]],
     [7, ['number', '0'], 0, 0, [0, None]]],
    'loadheapfromjournal':
    [[0, 'loadheap', 0, 0, [None, 1, None]],
     [1, 'journal', 0, 0, [0, None]]]}



#!/usr/bin/env python
#
# Copyright (c) 2011 Collabora Ltd. <http://www.collabora.co.uk/>
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place - Suite 330,
# Boston, MA 02111-1307, USA.

from gi.repository import GObject


class ConfigFile(GObject.GObject):

    """Load/save a simple (key = value) config file"""

    __gsignals__ = {
        'configuration-loaded': (GObject.SIGNAL_RUN_FIRST, GObject.TYPE_NONE,
                                 ()),
        'configuration-saved': (GObject.SIGNAL_RUN_FIRST, GObject.TYPE_NONE,
                                ()),
    }

    def __init__(self, config_file_path, valid_keys={}):
        GObject.GObject.__init__(self)

        self._config_file_path = config_file_path
        self._valid_keys = valid_keys
        self._config_hash = {}
        self._is_loaded = False

    def set_valid_keys(self, valid_keys):
        self._valid_keys = valid_keys

    def is_loaded(self):
        return self._is_loaded

    def get(self, key, empty_if_not_loaded=False):
        if key not in self._valid_keys:
            raise RuntimeError("Unknown config value %s" % key)

        if key in self._config_hash:
            value = self._config_hash[key]
        else:
            if self._valid_keys[key]["type"] == "text":
                value = ""
            elif self._valid_keys[key]["type"] == "boolean":
                value = False
            elif self._valid_keys[key]["type"] == "integer":
                value = 0

        return value

    def set(self, key, value):
        if key not in self._valid_keys:
            raise RuntimeError("Unknown config value %s" % key)

        self._config_hash[key] = value

    def load(self):
        try:
            config_file = open(self._config_file_path, 'r')
            lines = config_file.readlines()
            config_file.close()
            for line in lines:
                line = line.strip()
                k, v = line.split('=')
                k = k.strip(' ')
                v = v.strip(' ')
                if k not in self._valid_keys:
                    raise RuntimeError("Unknown config value %s" % k)
                value_type = self._valid_keys[k]["type"]
                if value_type == "text":
                    value = v
                elif value_type == "boolean":
                    value = eval(v)
                elif value_type == "integer":
                    value = int(v)
                self._config_hash[k] = value
            self._is_loaded = True
            self.emit('configuration-loaded')
        except Exception as e:
            print e

        return self._is_loaded

    def save(self):
        config_file = open(self._config_file_path, 'w')
        for k in self._config_hash.keys():
            v = self._config_hash[k]
            l = "%s = %s\n" % (k, v)
            config_file.write(l)
        config_file.close()
        self.emit('configuration-saved')

    def dump_keys(self):
        print "\n\nDumping keys\n\n"
        for k in self._config_hash.keys():
            v = self._config_hash[k]
            l = "%s = %s\n" % (k, v)
            print l


def test_save_load(test_config_file):
    keys = {}
    keys["nick"] = {"type": "text"}
    keys["account_id"] = {"type": "text"}
    keys["server"] = {"type": "text"}
    keys["port"] = {"type": "text"}
    keys["password"] = {"type": "text"}
    keys["register"] = {"type": "text"}

    c = ConfigFile(test_config_file)
    c.set_valid_keys(keys)
    c.set("nick", "rgs")
    c.set("account_id", "rgs@andromeda")
    c.set("server", "andromeda")
    c.set("port", 5223)
    c.set("password", "97c74fa0dc3b39b8c87f119fa53cced2b7040786")
    c.set("register", True)

    c.save()

    c = ConfigFile(test_config_file)
    c.set_valid_keys(keys)
    c.load()
    c.dump_keys()


def _configuration_saved_cb(config_file_obj):
    print "_configuration_saved_cb called"
    config_file_obj.dump_keys()


def _configuration_loaded_cb(config_file_obj):
    print "_configuration_loaded_cb called"
    config_file_obj.dump_keys()


def test_signals(test_config_file):
    keys = {}
    keys["nick"] = {"type": "text"}
    keys["account_id"] = {"type": "text"}
    keys["server"] = {"type": "text"}
    keys["port"] = {"type": "text"}
    keys["password"] = {"type": "text"}
    keys["register"] = {"type": "text"}

    c = ConfigFile(test_config_file)
    c.connect('configuration-saved', _configuration_saved_cb)
    c.set_valid_keys(keys)
    c.set("nick", "rgs")
    c.set("account_id", "rgs@andromeda")
    c.set("server", "andromeda")
    c.set("port", 5223)
    c.set("password", "97c74fa0dc3b39b8c87f119fa53cced2b7040786")
    c.set("register", True)

    c.save()

    c = ConfigFile(test_config_file)
    c.connect('configuration-loaded', _configuration_loaded_cb)
    c.set_valid_keys(keys)
    c.load()


if __name__ == "__main__":
    test_save_load("/tmp/configfile.0001")
    test_signals("/tmp/configfile.0002")



# Copyright (c) 2012, Walter Bender

# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA


def myblock(tw, x):  # ignore second argument
    ''' Load journal stats to heap (Sugar only) '''

    import os
    from gettext import gettext as _

    MAX = 19
    DIROFINTEREST = 'datastore'

    class ParseJournal():

        ''' Simple parser of datastore '''

        def __init__(self):
            self._dsdict = {}
            self._activity_name = []
            self._activity_count = []

            homepath = os.environ['HOME']
            for name in os.listdir(os.path.join(homepath, ".sugar")):
                path = os.path.join(homepath, ".sugar", name)
                if isdsdir(path):
                    self._dsdict[os.path.basename(path)] = []
                    if not os.path.isdir(path):
                        continue

                    for dsobjdir in os.listdir(path):
                        if len(dsobjdir) != DIROFINTEREST + 2:
                            continue

                        dsobjdir = os.path.join(path, dsobjdir)
                        if not os.path.isdir(dsobjdir):
                            continue

                        for dsobj in os.listdir(dsobjdir):
                            dsobj = os.path.join(dsobjdir, dsobj)

                            self._dsdict[os.path.basename(path)].append({})
                            activity = isactivity(dsobj)
                            if not activity:
                                self._dsdict[os.path.basename(path)][-1][
                                    'activity'] = 'media object'
                            else:
                                self._dsdict[os.path.basename(path)][-1][
                                    'activity'] = activity

            for k, v in self._dsdict.iteritems():
                for a in v:
                    if 'activity' in a:
                        if a['activity'] in self._activity_name:
                            i = self._activity_name.index(a['activity'])
                            self._activity_count[i] += 1
                        else:
                            self._activity_name.append(a['activity'])
                            self._activity_count.append(1)

        def get_sorted(self):
            activity_tuples = []
            for i in range(len(self._activity_name)):
                activity_tuples.append((self._activity_name[i],
                                        self._activity_count[i]))
            sorted_tuples = sorted(activity_tuples, key=lambda x: x[1])
            activity_list = []
            count = 0
            length = len(sorted_tuples)
            for i in range(length):
                if i < MAX:
                    activity_list.append([sorted_tuples[length - i - 1][0],
                                          sorted_tuples[length - i - 1][1]])
                else:
                    count += sorted_tuples[length - i - 1][1]
            if count > 0:
                activity_list.append([_('other'), count])
            return activity_list

    def hascomponent(path, component):
        ''' Return metadata attribute, if any '''
        if not os.path.exists(os.path.join(path, 'metadata')):
            return False
        if not os.path.exists(os.path.join(path, 'metadata', component)):
            return False
        fd = open(os.path.join(path, 'metadata', component))
        data = fd.readline()
        fd.close()
        if len(data) == 0:
            return False
        return data

    def isactivity(path):
        ''' Return activity name '''
        activity = hascomponent(path, 'activity')
        if not activity:
            return False
        else:
            return activity.split('.')[-1]

    def isdsdir(path):
        ''' Only interested if it is a datastore directory '''
        if not os.path.isdir(path):
            return False
        if not os.path.exists(os.path.join(path, DIROFINTEREST)):
            return False
        return True

    data = ParseJournal()
    activity_list = data.get_sorted()
    for a in activity_list:
        tw.lc.heap.append(a[0])
        tw.lc.heap.append(a[1])

    tw.lc.heap.append(activity_list[0][1])
    return



# -*- coding: utf-8 -*-
# Copyright (c) 2010-13 Walter Bender

# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:

# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.

# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.

import os
import cairo
import tempfile

from gi.repository import GLib
from gi.repository import Gdk
from random import uniform
from math import sin, cos, pi, sqrt
from .taconstants import (TURTLE_LAYER, DEFAULT_TURTLE_COLORS, DEFAULT_TURTLE,
                         Color, ColorObj)
from .tasprite_factory import SVG, svg_str_to_pixbuf
from .tacanvas import wrap100, COLOR_TABLE
from .sprites import Sprite
from .tautils import (debug_output, data_to_string, round_int, get_path,
                     image_to_base64)
from TurtleArt.talogo import logoerror

SHAPES = 36
DEGTOR = pi / 180.
RTODEG = 180. / pi


def generate_turtle_pixbufs(colors):
    ''' Generate pixbufs for generic turtles '''
    shapes = []
    svg = SVG()
    svg.set_scale(1.0)
    for i in range(SHAPES):
        svg.set_orientation(i * 10)
        shapes.append(svg_str_to_pixbuf(svg.turtle(colors)))
    return shapes


class Turtles:

    def __init__(self, turtle_window):
        ''' Class to hold turtles '''
        self.turtle_window = turtle_window
        self.sprite_list = turtle_window.sprite_list
        self.width = turtle_window.width
        self.height = turtle_window.height
        self.dict = {}
        self._default_pixbufs = []
        self._active_turtle = None
        self._default_turtle_name = DEFAULT_TURTLE

    def get_turtle(self, turtle_name, append=False, colors=None):
        ''' Find a turtle '''
        if turtle_name in self.dict:
            return self.dict[turtle_name]
        elif not append:
            return None
        else:
            if colors is None:
                Turtle(self, turtle_name)
            elif isinstance(colors, (list, tuple)):
                Turtle(self, turtle_name, colors)
            else:
                Turtle(self, turtle_name, colors.split(','))
            return self.dict[turtle_name]

    def get_turtle_key(self, turtle):
        ''' Find a turtle's name '''
        for turtle_name in iter(self.dict):
            if self.dict[turtle_name] == turtle:
                return turtle_name
        return None

    def turtle_count(self):
        ''' How many turtles are there? '''
        return(len(self.dict))

    def add_to_dict(self, turtle_name, turtle):
        ''' Add a new turtle '''
        self.dict[turtle_name] = turtle

    def remove_from_dict(self, turtle_name):
        ''' Delete a turtle '''
        if turtle_name in self.dict:
            del(self.dict[turtle_name])

    def show_all(self):
        ''' Make all turtles visible '''
        for turtle_name in iter(self.dict):
            self.dict[turtle_name].show()

    def spr_to_turtle(self, spr):
        ''' Find the turtle that corresponds to sprite spr. '''
        for turtle_name in iter(self.dict):
            if spr == self.dict[turtle_name].spr:
                return self.dict[turtle_name]
        return None

    def get_pixbufs(self):
        ''' Get the pixbufs for the default turtle shapes. '''
        if self._default_pixbufs == []:
            self._default_pixbufs = generate_turtle_pixbufs(
                ["#008000", "#00A000"])
        return(self._default_pixbufs)

    def turtle_to_screen_coordinates(self, pos):
        ''' The origin of turtle coordinates is the center of the screen '''
        return [self.width / 2.0 + pos[0], self._invert_y_coordinate(pos[1])]

    def screen_to_turtle_coordinates(self, pos):
        ''' The origin of the screen coordinates is the upper-left corner '''
        return [pos[0] - self.width / 2.0, self._invert_y_coordinate(pos[1])]

    def _invert_y_coordinate(self, y):
        ''' Positive y goes up in turtle coordinates, down in sceeen
        coordinates '''
        return self.height / 2.0 - y

    def reset_turtles(self):
        for turtle_name in iter(self.dict):
            self.set_turtle(turtle_name)
            if not self._active_turtle.get_remote():
                self._active_turtle.reset_turtle()
        self.set_turtle(self._default_turtle_name)

    def get_turtle_x(self, turtle_name):
        if turtle_name not in self.dict:
            debug_output('%s not found in turtle dictionary' % (turtle_name),
                         self.turtle_window.running_sugar)
            raise logoerror("#syntaxerror")
        return self.dict[turtle_name].get_x()

    def get_turtle_y(self, turtle_name):
        if turtle_name not in self.dict:
            debug_output('%s not found in turtle dictionary' % (turtle_name),
                         self.turtle_window.running_sugar)
            raise logoerror("#syntaxerror")
        return self.dict[turtle_name].get_y()

    def get_turtle_heading(self, turtle_name):
        if turtle_name not in self.dict:
            debug_output('%s not found in turtle dictionary' % (turtle_name),
                         self.turtle_window.running_sugar)
            raise logoerror("#syntaxerror")
        return self.dict[turtle_name].get_heading()

    def set_turtle(self, turtle_name, colors=None):
        ''' Select the current turtle and associated pen status '''
        if turtle_name not in self.dict:
            # if it is a new turtle, start it in the center of the screen
            self._active_turtle = self.get_turtle(turtle_name, True, colors)
            self._active_turtle.set_heading(0.0, False)
            self._active_turtle.set_xy(0.0, 0.0, share=False, pendown=False)
            self._active_turtle.set_pen_state(True)
        elif colors is not None:
            self._active_turtle = self.get_turtle(turtle_name, False)
            self._active_turtle.set_turtle_colors(colors)
        else:
            self._active_turtle = self.get_turtle(turtle_name, False)
        self._active_turtle.show()
        # self._active_turtle.set_color(share=False)
        # self._active_turtle.set_gray(share=False)
        # self._active_turtle.set_shade(share=False)
        # self._active_turtle.set_pen_size(share=False)
        # self._active_turtle.set_pen_state(share=False)

        # self._active_turtle.set_xy(share=False)
        # self._active_turtle.set_heading(share=False)

    def set_default_turtle_name(self, name):
        self._default_turtle_name = name

    def get_default_turtle_name(self):
        return self._default_turtle_name

    def set_active_turtle(self, active_turtle):
        self._active_turtle = active_turtle

    def get_active_turtle(self):
        return self._active_turtle


class Turtle:

    def __init__(self, turtles, turtle_name, turtle_colors=None):
        ''' The turtle is not a block, just a sprite with an orientation '''
        self.spr = None
        self.label_block = None
        self._turtles = turtles
        self._shapes = []
        self._custom_shapes = False
        self._name = turtle_name
        self._hidden = False
        self._remote = False
        self._x = 0.0
        self._y = 0.0
        self._heading = 0.0
        self._half_width = 0
        self._half_height = 0
        self._drag_radius = None
        self._pen_shade = 50
        self._pen_color = 0
        self._pen_gray = 100
        if self._turtles.turtle_window.coord_scale == 1:
            self._pen_size = 5
        else:
            self._pen_size = 1
        self._pen_state = True
        self._pen_fill = False
        self._poly_points = []

        self._prep_shapes(turtle_name, self._turtles, turtle_colors)

        # Create a sprite for the turtle in interactive mode.
        if turtles.sprite_list is not None:
            self.spr = Sprite(self._turtles.sprite_list, 0, 0, self._shapes[0])

            self._calculate_sizes()

            # Choose a random angle from which to attach the turtle
            # label to be used when sharing.
            angle = uniform(0, pi * 4 / 3.0)  # 240 degrees
            width = self._shapes[0].get_width()
            radius = width * 0.67
            # Restrict the angle to the sides: 30-150; 210-330
            if angle > pi * 2 / 3.0:
                angle += pi / 2.0  # + 90
                self.label_xy = [int(radius * sin(angle)),
                                 int(radius * cos(angle) + width / 2.0)]
            else:
                angle += pi / 6.0  # + 30
                self.label_xy = [int(radius * sin(angle) + width / 2.0),
                                 int(radius * cos(angle) + width / 2.0)]

        self._turtles.add_to_dict(turtle_name, self)

    def _calculate_sizes(self):
        self._half_width = int(self.spr.rect.width / 2.0)
        self._half_height = int(self.spr.rect.height / 2.0)
        self._drag_radius = ((self._half_width * self._half_width) +
                             (self._half_height * self._half_height)) / 6

    def set_remote(self):
        self._remote = True

    def get_remote(self):
        return self._remote

    def _prep_shapes(self, name, turtles=None, turtle_colors=None):
        # If the turtle name is an int, we'll use a palette color as the
        # turtle color
        try:
            int_key = int(name)
            use_color_table = True
        except ValueError:
            use_color_table = False

        if turtle_colors is not None:
            self.colors = turtle_colors[:]
            self._shapes = generate_turtle_pixbufs(self.colors)
        elif use_color_table:
            fill = wrap100(int_key)
            stroke = wrap100(fill + 10)
            self.colors = ['#%06x' % (COLOR_TABLE[fill]),
                           '#%06x' % (COLOR_TABLE[stroke])]
            self._shapes = generate_turtle_pixbufs(self.colors)
        else:
            if turtles is not None:
                self.colors = DEFAULT_TURTLE_COLORS
                self._shapes = turtles.get_pixbufs()

    def set_turtle_colors(self, turtle_colors):
        ''' reset the colors of a preloaded turtle '''
        if turtle_colors is not None:
            self.colors = turtle_colors[:]
            self._shapes = generate_turtle_pixbufs(self.colors)
            self.set_heading(self._heading, share=False)

    def set_shapes(self, shapes, i=0):
        ''' Reskin the turtle '''
        n = len(shapes)
        if n == 1 and i > 0:  # set shape[i]
            if i < len(self._shapes):
                self._shapes[i] = shapes[0]
        elif n == SHAPES:  # all shapes have been precomputed
            self._shapes = shapes[:]
        else:  # rotate shapes
            if n != 1:
                debug_output("%d images passed to set_shapes: ignoring" % (n),
                             self._turtles.turtle_window.running_sugar)
            if self._heading == 0.0:  # rotate the shapes
                images = []
                w, h = shapes[0].get_width(), shapes[0].get_height()
                nw = nh = int(sqrt(w * w + h * h))
                for i in range(SHAPES):
                    surface = cairo.ImageSurface(cairo.FORMAT_ARGB32, nw, nh)
                    context = cairo.Context(surface)

                    context.translate(nw / 2.0, nh / 2.0)
                    context.rotate(i * 10 * pi / 180.)
                    context.translate(-nw / 2.0, -nh / 2.0)
                    Gdk.cairo_set_source_pixbuf(
                        context, shapes[0], (nw - w) / 2.0, (nh - h) / 2.0)
                    context.rectangle(0, 0, nw, nh)
                    context.fill()
                    images.append(surface)
                self._shapes = images[:]
            else:  # associate shape with image at current heading
                j = int(self._heading + 5) % 360 / (360 / SHAPES)
                self._shapes[j] = shapes[0]
        self._custom_shapes = True
        self.show()
        self._calculate_sizes()

    def reset_turtle(self):
        self.set_color(0)
        self.set_shade(50)
        self.set_gray(100)
        if self._turtles.turtle_window.coord_scale == 1:
            self.set_pen_size(5)
        else:
            self.set_pen_size(1)
        self.reset_shapes()
        self.set_heading(0.0)
        self.set_pen_state(False)
        self.move_turtle((0.0, 0.0))

        if self._turtles.turtle_window.sharing():
            event = data_to_string([self._turtles.turtle_window.nick,
                                    [0, 0]])
            self._turtles.turtle_window.send_event('x', event)

        self.set_pen_state(True)
        self.set_fill(False)
        self.hide()

    def reset_shapes(self):
        ''' Reset the shapes to the standard turtle '''
        if self._custom_shapes:
            self._shapes = generate_turtle_pixbufs(self.colors)
            self._custom_shapes = False
            self._calculate_sizes()

    def set_heading(self, heading=None, share=True):
        ''' Set the turtle heading (one shape per 360/SHAPES degrees) '''
        if heading is not None:
            self._heading = heading
        self._heading %= 360

        self._update_sprite_heading()

        if self._turtles.turtle_window.sharing() and share:
            event = data_to_string([self._turtles.turtle_window.nick,
                                     round_int(self._heading)])
            self._turtles.turtle_window.send_event('r', event)

    def _update_sprite_heading(self):
        ''' Update the sprite to reflect the current heading '''
        i = (int(self._heading + 5) % 360) // (360 // SHAPES)
        if not self._hidden and self.spr is not None:
            try:
                self.spr.set_shape(self._shapes[i])
            except IndexError:
                self.spr.set_shape(self._shapes[0])

    def set_color(self, color=None, share=True):
        ''' Set the pen color for this turtle. '''
        if isinstance(color, ColorObj):
            # See comment in tatype.py TYPE_BOX -> TYPE_COLOR
            color = color.color
        if color is None:
            color = self._pen_color
        # Special case for color blocks from CONSTANTS
        elif isinstance(color, Color):
            self.set_shade(color.shade, share)
            self.set_gray(color.gray, share)
            if color.color is not None:
                color = color.color
            else:
                color = self._pen_color

        self._pen_color = color

        self._turtles.turtle_window.canvas.set_fgcolor(shade=self._pen_shade,
                                                       gray=self._pen_gray,
                                                       color=self._pen_color)

        if self._turtles.turtle_window.sharing() and share:
            event = data_to_string([self._turtles.turtle_window.nick,
                                    round_int(self._pen_color)])
            self._turtles.turtle_window.send_event('c', event)

    def set_gray(self, gray=None, share=True):
        ''' Set the pen gray level for this turtle. '''
        if gray is not None:
            self._pen_gray = gray

        if self._pen_gray < 0:
            self._pen_gray = 0
        if self._pen_gray > 100:
            self._pen_gray = 100

        self._turtles.turtle_window.canvas.set_fgcolor(shade=self._pen_shade,
                                                       gray=self._pen_gray,
                                                       color=self._pen_color)

        if self._turtles.turtle_window.sharing() and share:
            event = data_to_string([self._turtles.turtle_window.nick,
                                    round_int(self._pen_gray)])
            self._turtles.turtle_window.send_event('g', event)

    def set_shade(self, shade=None, share=True):
        ''' Set the pen shade for this turtle. '''
        if shade is not None:
            self._pen_shade = shade

        self._turtles.turtle_window.canvas.set_fgcolor(shade=self._pen_shade,
                                                       gray=self._pen_gray,
                                                       color=self._pen_color)

        if self._turtles.turtle_window.sharing() and share:
            event = data_to_string([self._turtles.turtle_window.nick,
                                    round_int(self._pen_shade)])
            self._turtles.turtle_window.send_event('s', event)

    def set_pen_size(self, pen_size=None, share=True):
        ''' Set the pen size for this turtle. '''
        if pen_size is not None:
            self._pen_size = max(0, pen_size)

        self._turtles.turtle_window.canvas.set_pen_size(
            self._pen_size * self._turtles.turtle_window.coord_scale)

        if self._turtles.turtle_window.sharing() and share:
            event = data_to_string([self._turtles.turtle_window.nick,
                                    round_int(self._pen_size)])
            self._turtles.turtle_window.send_event('w', event)

    def set_pen_state(self, pen_state=None, share=True):
        ''' Set the pen state (down==True) for this turtle. '''
        if pen_state is not None:
            self._pen_state = pen_state

        if self._turtles.turtle_window.sharing() and share:
            event = data_to_string([self._turtles.turtle_window.nick,
                                    self._pen_state])
            self._turtles.turtle_window.send_event('p', event)

    def set_fill(self, state=False):
        self._pen_fill = state
        if not self._pen_fill:
            self._poly_points = []

    def set_poly_points(self, poly_points=None):
        if poly_points is not None:
            self._poly_points = poly_points[:]

    def start_fill(self):
        self._pen_fill = True
        self._poly_points = []

    def stop_fill(self, share=True):
        self._pen_fill = False
        if len(self._poly_points) == 0:
            return

        self._turtles.turtle_window.canvas.fill_polygon(self._poly_points)

        if self._turtles.turtle_window.sharing() and share:
            shared_poly_points = []
            for p in self._poly_points:
                x, y = self._turtles.turtle_to_screen_coordinates(
                    (p[1], p[2]))
                if p[0] in ['move', 'line']:
                    shared_poly_points.append((p[0], x, y))
                elif p[0] in ['rarc', 'larc']:
                    shared_poly_points.append((p[0], x, y, p[3], p[4], p[5]))
                event = data_to_string([self._turtles.turtle_window.nick,
                                        shared_poly_points])
                self._turtles.turtle_window.send_event('F', event)
        self._poly_points = []

    def hide(self):
        if self.spr is not None:
            self.spr.hide()
        if self.label_block is not None:
            self.label_block.spr.hide()
        self._hidden = True

    def show(self):
        if self.spr is not None:
            self.spr.set_layer(TURTLE_LAYER)
            self._hidden = False
        self.move_turtle_spr((self._x, self._y))
        self.set_heading(self._heading, share=False)
        if self.label_block is not None:
            self.label_block.spr.set_layer(TURTLE_LAYER + 1)

    def move_turtle(self, pos=None):
        ''' Move the turtle's position '''
        if pos is None:
            pos = self.get_xy()

        self._x, self._y = pos[0], pos[1]
        if self.spr is not None:
            self.move_turtle_spr(pos)

    def move_turtle_spr(self, pos):
        ''' Move the turtle's sprite '''
        pos = self._turtles.turtle_to_screen_coordinates(pos)

        pos[0] -= self._half_width
        pos[1] -= self._half_height

        if not self._hidden and self.spr is not None:
            self.spr.move(pos)
        if self.label_block is not None:
            self.label_block.spr.move((pos[0] + self.label_xy[0],
                                       pos[1] + self.label_xy[1]))

    def right(self, degrees, share=True):
        ''' Rotate turtle clockwise '''
        self._heading += degrees
        self._heading %= 360

        self._update_sprite_heading()

        if self._turtles.turtle_window.sharing() and share:
            event = data_to_string([self._turtles.turtle_window.nick,
                                    round_int(self._heading)])
            self._turtles.turtle_window.send_event('r', event)

    def left(self, degrees, share=True):
        degrees = 0 - degrees
        self.right(degrees, share)

    def _draw_line(self, old, new, pendown):
        if self._pen_state and pendown:
            self._turtles.turtle_window.canvas.set_source_rgb()
            pos1 = self._turtles.turtle_to_screen_coordinates(old)
            pos2 = self._turtles.turtle_to_screen_coordinates(new)
            self._turtles.turtle_window.canvas.draw_line(pos1[0], pos1[1],
                                                         pos2[0], pos2[1])
            if self._pen_fill:
                if self._poly_points == []:
                    self._poly_points.append(('move', pos1[0], pos1[1]))
                self._poly_points.append(('line', pos2[0], pos2[1]))

    def forward(self, distance, share=True):
        scaled_distance = distance * self._turtles.turtle_window.coord_scale

        old = self.get_xy()
        xcor = old[0] + scaled_distance * sin(self._heading * DEGTOR)
        ycor = old[1] + scaled_distance * cos(self._heading * DEGTOR)

        self._draw_line(old, (xcor, ycor), True)
        self.move_turtle((xcor, ycor))

        if self._turtles.turtle_window.sharing() and share:
            event = data_to_string([self._turtles.turtle_window.nick,
                                    int(distance)])
            self._turtles.turtle_window.send_event('f', event)

    def backward(self, distance, share=True):
        distance = 0 - distance
        self.forward(distance, share)

    def set_xy(self, x=None, y=None, share=True, pendown=True, dragging=False):
        old = self.get_xy()
        if x is None or y is None:
            x = old[0]
            y = old[1]
        if dragging:
            xcor = x
            ycor = y
        else:
            xcor = x * self._turtles.turtle_window.coord_scale
            ycor = y * self._turtles.turtle_window.coord_scale

        self._draw_line(old, (xcor, ycor), pendown)
        self.move_turtle((xcor, ycor))

        if self._turtles.turtle_window.sharing() and share:
            event = data_to_string([self._turtles.turtle_window.nick,
                                    [round_int(xcor), round_int(ycor)]])
            self._turtles.turtle_window.send_event('x', event)

    def arc(self, a, r, share=True):
        ''' Draw an arc '''
        if self._pen_state:
            self._turtles.turtle_window.canvas.set_source_rgb()
        if a < 0:
            pos = self.larc(-a, r)
        else:
            pos = self.rarc(a, r)

        self.move_turtle(pos)

        if self._turtles.turtle_window.sharing() and share:
            event = data_to_string([self._turtles.turtle_window.nick,
                                    [round_int(a), round_int(r)]])
            self._turtles.turtle_window.send_event('a', event)

    def rarc(self, a, r):
        ''' draw a clockwise arc '''
        r *= self._turtles.turtle_window.coord_scale
        if r < 0:
            r = -r
            a = -a
        pos = self.get_xy()
        cx = pos[0] + r * cos(self._heading * DEGTOR)
        cy = pos[1] - r * sin(self._heading * DEGTOR)
        if self._pen_state:
            npos = self._turtles.turtle_to_screen_coordinates((cx, cy))
            self._turtles.turtle_window.canvas.rarc(npos[0], npos[1], r, a,
                                                    self._heading)

            if self._pen_fill:
                self._poly_points.append(('move', npos[0], npos[1]))
                self._poly_points.append(('rarc', npos[0], npos[1], r,
                                          (self._heading - 180) * DEGTOR,
                                          (self._heading - 180 + a) * DEGTOR))

        self.right(a, False)
        return [cx - r * cos(self._heading * DEGTOR),
                cy + r * sin(self._heading * DEGTOR)]

    def larc(self, a, r):
        ''' draw a counter-clockwise arc '''
        r *= self._turtles.turtle_window.coord_scale
        if r < 0:
            r = -r
            a = -a
        pos = self.get_xy()
        cx = pos[0] - r * cos(self._heading * DEGTOR)
        cy = pos[1] + r * sin(self._heading * DEGTOR)
        if self._pen_state:
            npos = self._turtles.turtle_to_screen_coordinates((cx, cy))
            self._turtles.turtle_window.canvas.larc(npos[0], npos[1], r, a,
                                                    self._heading)

            if self._pen_fill:
                self._poly_points.append(('move', npos[0], npos[1]))
                self._poly_points.append(('larc', npos[0], npos[1], r,
                                          (self._heading) * DEGTOR,
                                          (self._heading - a) * DEGTOR))

        self.right(-a, False)
        return [cx + r * cos(self._heading * DEGTOR),
                cy - r * sin(self._heading * DEGTOR)]

    def draw_pixbuf(self, pixbuf, a, b, x, y, w, h, path, share=True):
        ''' Draw a pixbuf '''
        self._turtles.turtle_window.canvas.draw_pixbuf(
            pixbuf, a, b, x, y, w, h, self._heading)

        if self._turtles.turtle_window.sharing() and share:
            if self._turtles.turtle_window.running_sugar:
                tmp_path = get_path(self._turtles.turtle_window.activity,
                                    'instance')
            else:
                tmp_path = tempfile.gettempdir()
            tmp_file = os.path.join(
                get_path(self._turtles.turtle_window.activity, 'instance'),
                'tmpfile.png')
            pixbuf.save(tmp_file, 'png', {'quality': '100'})
            data = image_to_base64(tmp_file, tmp_path)
            height = pixbuf.get_height()
            width = pixbuf.get_width()

            pos = self._turtles.screen_to_turtle_coordinates((x, y))

            event = data_to_string([self._turtles.turtle_window.nick,
                                    [round_int(a), round_int(b),
                                     round_int(pos[0]),
                                     round_int(pos[1]),
                                     round_int(w), round_int(h),
                                     round_int(width),
                                     round_int(height),
                                     data]])

            def send_event():
                self._turtles.turtle_window.send_event('P', event)
                return False

            GLib.idle_add(send_event)

            os.remove(tmp_file)

    def draw_text(self, label, x, y, size, w, share=True):
        ''' Draw text '''
        self._turtles.turtle_window.canvas.draw_text(
            label, x, y, size, w, self._heading,
            self._turtles.turtle_window.coord_scale)

        if self._turtles.turtle_window.sharing() and share:
            event = data_to_string([self._turtles.turtle_window.nick,
                                    [label, round_int(x), round_int(y),
                                     round_int(size), round_int(w)]])
            self._turtles.turtle_window.send_event('W', event)

    def read_pixel(self):
        """ Read r, g, b, a from the canvas and push b, g, r to the stack """
        r, g, b, a = self.get_pixel()
        self._turtles.turtle_window.lc.heap.append(b)
        self._turtles.turtle_window.lc.heap.append(g)
        self._turtles.turtle_window.lc.heap.append(r)

    def get_color_index(self):
        r, g, b, a = self.get_pixel()
        color_index = self._turtles.turtle_window.canvas.get_color_index(
            r, g, b)
        return color_index

    def get_name(self):
        return self._name

    def get_xy(self):
        return [self._x, self._y]

    def get_x(self):
        return self._x

    def get_y(self):
        return self._y

    def get_heading(self):
        return self._heading

    def get_color(self):
        return self._pen_color

    def get_gray(self):
        return self._pen_gray

    def get_shade(self):
        return self._pen_shade

    def get_pen_size(self):
        return self._pen_size

    def get_pen_state(self):
        return self._pen_state

    def get_fill(self):
        return self._pen_fill

    def get_poly_points(self):
        return self._poly_points

    def get_pixel(self):
        pos = self._turtles.turtle_to_screen_coordinates(self.get_xy())
        return self._turtles.turtle_window.canvas.get_pixel(pos[0], pos[1])

    def get_drag_radius(self):
        if self._drag_radius is None:
            self._calculate_sizes()
        return self._drag_radius
